# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = ['AutopilotConfigArgs', 'AutopilotConfig']

@pulumi.input_type
class AutopilotConfigArgs:
    def __init__(__self__, *,
                 cleanup_dead_servers: Optional[pulumi.Input[bool]] = None,
                 datacenter: Optional[pulumi.Input[str]] = None,
                 disable_upgrade_migration: Optional[pulumi.Input[bool]] = None,
                 last_contact_threshold: Optional[pulumi.Input[str]] = None,
                 max_trailing_logs: Optional[pulumi.Input[int]] = None,
                 redundancy_zone_tag: Optional[pulumi.Input[str]] = None,
                 server_stabilization_time: Optional[pulumi.Input[str]] = None,
                 upgrade_version_tag: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a AutopilotConfig resource.
        :param pulumi.Input[bool] cleanup_dead_servers: Whether to remove failing servers when a
               replacement comes online. Defaults to true.
        :param pulumi.Input[str] datacenter: The datacenter to use. This overrides the agent's
               default datacenter and the datacenter in the provider setup.
        :param pulumi.Input[bool] disable_upgrade_migration: Whether to disable [upgrade migrations](https://www.consul.io/docs/guides/autopilot.html#redundancy-zones).
               Defaults to false.
        :param pulumi.Input[str] last_contact_threshold: The time after which a server is
               considered as unhealthy and will be removed. Defaults to `"200ms"`.
        :param pulumi.Input[int] max_trailing_logs: The maximum number of Raft log entries a
               server can trail the leader. Defaults to 250.
        :param pulumi.Input[str] redundancy_zone_tag: The [redundancy zone](https://www.consul.io/docs/guides/autopilot.html#redundancy-zones)
               tag to use. Consul will try to keep one voting server by zone to take advantage
               of isolated failure domains. Defaults to an empty string.
        :param pulumi.Input[str] server_stabilization_time: The period to wait for a server to be
               healthy and stable before being promoted to a full, voting member. Defaults to
               `"10s"`.
        :param pulumi.Input[str] upgrade_version_tag: The tag to override the version information
               used during a migration. Defaults to an empty string.
        """
        if cleanup_dead_servers is not None:
            pulumi.set(__self__, "cleanup_dead_servers", cleanup_dead_servers)
        if datacenter is not None:
            pulumi.set(__self__, "datacenter", datacenter)
        if disable_upgrade_migration is not None:
            pulumi.set(__self__, "disable_upgrade_migration", disable_upgrade_migration)
        if last_contact_threshold is not None:
            pulumi.set(__self__, "last_contact_threshold", last_contact_threshold)
        if max_trailing_logs is not None:
            pulumi.set(__self__, "max_trailing_logs", max_trailing_logs)
        if redundancy_zone_tag is not None:
            pulumi.set(__self__, "redundancy_zone_tag", redundancy_zone_tag)
        if server_stabilization_time is not None:
            pulumi.set(__self__, "server_stabilization_time", server_stabilization_time)
        if upgrade_version_tag is not None:
            pulumi.set(__self__, "upgrade_version_tag", upgrade_version_tag)

    @property
    @pulumi.getter(name="cleanupDeadServers")
    def cleanup_dead_servers(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether to remove failing servers when a
        replacement comes online. Defaults to true.
        """
        return pulumi.get(self, "cleanup_dead_servers")

    @cleanup_dead_servers.setter
    def cleanup_dead_servers(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "cleanup_dead_servers", value)

    @property
    @pulumi.getter
    def datacenter(self) -> Optional[pulumi.Input[str]]:
        """
        The datacenter to use. This overrides the agent's
        default datacenter and the datacenter in the provider setup.
        """
        return pulumi.get(self, "datacenter")

    @datacenter.setter
    def datacenter(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "datacenter", value)

    @property
    @pulumi.getter(name="disableUpgradeMigration")
    def disable_upgrade_migration(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether to disable [upgrade migrations](https://www.consul.io/docs/guides/autopilot.html#redundancy-zones).
        Defaults to false.
        """
        return pulumi.get(self, "disable_upgrade_migration")

    @disable_upgrade_migration.setter
    def disable_upgrade_migration(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "disable_upgrade_migration", value)

    @property
    @pulumi.getter(name="lastContactThreshold")
    def last_contact_threshold(self) -> Optional[pulumi.Input[str]]:
        """
        The time after which a server is
        considered as unhealthy and will be removed. Defaults to `"200ms"`.
        """
        return pulumi.get(self, "last_contact_threshold")

    @last_contact_threshold.setter
    def last_contact_threshold(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "last_contact_threshold", value)

    @property
    @pulumi.getter(name="maxTrailingLogs")
    def max_trailing_logs(self) -> Optional[pulumi.Input[int]]:
        """
        The maximum number of Raft log entries a
        server can trail the leader. Defaults to 250.
        """
        return pulumi.get(self, "max_trailing_logs")

    @max_trailing_logs.setter
    def max_trailing_logs(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "max_trailing_logs", value)

    @property
    @pulumi.getter(name="redundancyZoneTag")
    def redundancy_zone_tag(self) -> Optional[pulumi.Input[str]]:
        """
        The [redundancy zone](https://www.consul.io/docs/guides/autopilot.html#redundancy-zones)
        tag to use. Consul will try to keep one voting server by zone to take advantage
        of isolated failure domains. Defaults to an empty string.
        """
        return pulumi.get(self, "redundancy_zone_tag")

    @redundancy_zone_tag.setter
    def redundancy_zone_tag(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "redundancy_zone_tag", value)

    @property
    @pulumi.getter(name="serverStabilizationTime")
    def server_stabilization_time(self) -> Optional[pulumi.Input[str]]:
        """
        The period to wait for a server to be
        healthy and stable before being promoted to a full, voting member. Defaults to
        `"10s"`.
        """
        return pulumi.get(self, "server_stabilization_time")

    @server_stabilization_time.setter
    def server_stabilization_time(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "server_stabilization_time", value)

    @property
    @pulumi.getter(name="upgradeVersionTag")
    def upgrade_version_tag(self) -> Optional[pulumi.Input[str]]:
        """
        The tag to override the version information
        used during a migration. Defaults to an empty string.
        """
        return pulumi.get(self, "upgrade_version_tag")

    @upgrade_version_tag.setter
    def upgrade_version_tag(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "upgrade_version_tag", value)


@pulumi.input_type
class _AutopilotConfigState:
    def __init__(__self__, *,
                 cleanup_dead_servers: Optional[pulumi.Input[bool]] = None,
                 datacenter: Optional[pulumi.Input[str]] = None,
                 disable_upgrade_migration: Optional[pulumi.Input[bool]] = None,
                 last_contact_threshold: Optional[pulumi.Input[str]] = None,
                 max_trailing_logs: Optional[pulumi.Input[int]] = None,
                 redundancy_zone_tag: Optional[pulumi.Input[str]] = None,
                 server_stabilization_time: Optional[pulumi.Input[str]] = None,
                 upgrade_version_tag: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering AutopilotConfig resources.
        :param pulumi.Input[bool] cleanup_dead_servers: Whether to remove failing servers when a
               replacement comes online. Defaults to true.
        :param pulumi.Input[str] datacenter: The datacenter to use. This overrides the agent's
               default datacenter and the datacenter in the provider setup.
        :param pulumi.Input[bool] disable_upgrade_migration: Whether to disable [upgrade migrations](https://www.consul.io/docs/guides/autopilot.html#redundancy-zones).
               Defaults to false.
        :param pulumi.Input[str] last_contact_threshold: The time after which a server is
               considered as unhealthy and will be removed. Defaults to `"200ms"`.
        :param pulumi.Input[int] max_trailing_logs: The maximum number of Raft log entries a
               server can trail the leader. Defaults to 250.
        :param pulumi.Input[str] redundancy_zone_tag: The [redundancy zone](https://www.consul.io/docs/guides/autopilot.html#redundancy-zones)
               tag to use. Consul will try to keep one voting server by zone to take advantage
               of isolated failure domains. Defaults to an empty string.
        :param pulumi.Input[str] server_stabilization_time: The period to wait for a server to be
               healthy and stable before being promoted to a full, voting member. Defaults to
               `"10s"`.
        :param pulumi.Input[str] upgrade_version_tag: The tag to override the version information
               used during a migration. Defaults to an empty string.
        """
        if cleanup_dead_servers is not None:
            pulumi.set(__self__, "cleanup_dead_servers", cleanup_dead_servers)
        if datacenter is not None:
            pulumi.set(__self__, "datacenter", datacenter)
        if disable_upgrade_migration is not None:
            pulumi.set(__self__, "disable_upgrade_migration", disable_upgrade_migration)
        if last_contact_threshold is not None:
            pulumi.set(__self__, "last_contact_threshold", last_contact_threshold)
        if max_trailing_logs is not None:
            pulumi.set(__self__, "max_trailing_logs", max_trailing_logs)
        if redundancy_zone_tag is not None:
            pulumi.set(__self__, "redundancy_zone_tag", redundancy_zone_tag)
        if server_stabilization_time is not None:
            pulumi.set(__self__, "server_stabilization_time", server_stabilization_time)
        if upgrade_version_tag is not None:
            pulumi.set(__self__, "upgrade_version_tag", upgrade_version_tag)

    @property
    @pulumi.getter(name="cleanupDeadServers")
    def cleanup_dead_servers(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether to remove failing servers when a
        replacement comes online. Defaults to true.
        """
        return pulumi.get(self, "cleanup_dead_servers")

    @cleanup_dead_servers.setter
    def cleanup_dead_servers(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "cleanup_dead_servers", value)

    @property
    @pulumi.getter
    def datacenter(self) -> Optional[pulumi.Input[str]]:
        """
        The datacenter to use. This overrides the agent's
        default datacenter and the datacenter in the provider setup.
        """
        return pulumi.get(self, "datacenter")

    @datacenter.setter
    def datacenter(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "datacenter", value)

    @property
    @pulumi.getter(name="disableUpgradeMigration")
    def disable_upgrade_migration(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether to disable [upgrade migrations](https://www.consul.io/docs/guides/autopilot.html#redundancy-zones).
        Defaults to false.
        """
        return pulumi.get(self, "disable_upgrade_migration")

    @disable_upgrade_migration.setter
    def disable_upgrade_migration(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "disable_upgrade_migration", value)

    @property
    @pulumi.getter(name="lastContactThreshold")
    def last_contact_threshold(self) -> Optional[pulumi.Input[str]]:
        """
        The time after which a server is
        considered as unhealthy and will be removed. Defaults to `"200ms"`.
        """
        return pulumi.get(self, "last_contact_threshold")

    @last_contact_threshold.setter
    def last_contact_threshold(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "last_contact_threshold", value)

    @property
    @pulumi.getter(name="maxTrailingLogs")
    def max_trailing_logs(self) -> Optional[pulumi.Input[int]]:
        """
        The maximum number of Raft log entries a
        server can trail the leader. Defaults to 250.
        """
        return pulumi.get(self, "max_trailing_logs")

    @max_trailing_logs.setter
    def max_trailing_logs(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "max_trailing_logs", value)

    @property
    @pulumi.getter(name="redundancyZoneTag")
    def redundancy_zone_tag(self) -> Optional[pulumi.Input[str]]:
        """
        The [redundancy zone](https://www.consul.io/docs/guides/autopilot.html#redundancy-zones)
        tag to use. Consul will try to keep one voting server by zone to take advantage
        of isolated failure domains. Defaults to an empty string.
        """
        return pulumi.get(self, "redundancy_zone_tag")

    @redundancy_zone_tag.setter
    def redundancy_zone_tag(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "redundancy_zone_tag", value)

    @property
    @pulumi.getter(name="serverStabilizationTime")
    def server_stabilization_time(self) -> Optional[pulumi.Input[str]]:
        """
        The period to wait for a server to be
        healthy and stable before being promoted to a full, voting member. Defaults to
        `"10s"`.
        """
        return pulumi.get(self, "server_stabilization_time")

    @server_stabilization_time.setter
    def server_stabilization_time(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "server_stabilization_time", value)

    @property
    @pulumi.getter(name="upgradeVersionTag")
    def upgrade_version_tag(self) -> Optional[pulumi.Input[str]]:
        """
        The tag to override the version information
        used during a migration. Defaults to an empty string.
        """
        return pulumi.get(self, "upgrade_version_tag")

    @upgrade_version_tag.setter
    def upgrade_version_tag(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "upgrade_version_tag", value)


class AutopilotConfig(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 cleanup_dead_servers: Optional[pulumi.Input[bool]] = None,
                 datacenter: Optional[pulumi.Input[str]] = None,
                 disable_upgrade_migration: Optional[pulumi.Input[bool]] = None,
                 last_contact_threshold: Optional[pulumi.Input[str]] = None,
                 max_trailing_logs: Optional[pulumi.Input[int]] = None,
                 redundancy_zone_tag: Optional[pulumi.Input[str]] = None,
                 server_stabilization_time: Optional[pulumi.Input[str]] = None,
                 upgrade_version_tag: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Provides access to the [Autopilot Configuration](https://www.consul.io/docs/guides/autopilot.html)
        of Consul to automatically manage Consul servers.

        It includes to automatically cleanup dead servers, monitor the status of the Raft
        cluster and stable server introduction.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_consul as consul

        config = consul.AutopilotConfig("config",
            cleanup_dead_servers=False,
            last_contact_threshold="1s",
            max_trailing_logs=500)
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] cleanup_dead_servers: Whether to remove failing servers when a
               replacement comes online. Defaults to true.
        :param pulumi.Input[str] datacenter: The datacenter to use. This overrides the agent's
               default datacenter and the datacenter in the provider setup.
        :param pulumi.Input[bool] disable_upgrade_migration: Whether to disable [upgrade migrations](https://www.consul.io/docs/guides/autopilot.html#redundancy-zones).
               Defaults to false.
        :param pulumi.Input[str] last_contact_threshold: The time after which a server is
               considered as unhealthy and will be removed. Defaults to `"200ms"`.
        :param pulumi.Input[int] max_trailing_logs: The maximum number of Raft log entries a
               server can trail the leader. Defaults to 250.
        :param pulumi.Input[str] redundancy_zone_tag: The [redundancy zone](https://www.consul.io/docs/guides/autopilot.html#redundancy-zones)
               tag to use. Consul will try to keep one voting server by zone to take advantage
               of isolated failure domains. Defaults to an empty string.
        :param pulumi.Input[str] server_stabilization_time: The period to wait for a server to be
               healthy and stable before being promoted to a full, voting member. Defaults to
               `"10s"`.
        :param pulumi.Input[str] upgrade_version_tag: The tag to override the version information
               used during a migration. Defaults to an empty string.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[AutopilotConfigArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides access to the [Autopilot Configuration](https://www.consul.io/docs/guides/autopilot.html)
        of Consul to automatically manage Consul servers.

        It includes to automatically cleanup dead servers, monitor the status of the Raft
        cluster and stable server introduction.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_consul as consul

        config = consul.AutopilotConfig("config",
            cleanup_dead_servers=False,
            last_contact_threshold="1s",
            max_trailing_logs=500)
        ```

        :param str resource_name: The name of the resource.
        :param AutopilotConfigArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(AutopilotConfigArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 cleanup_dead_servers: Optional[pulumi.Input[bool]] = None,
                 datacenter: Optional[pulumi.Input[str]] = None,
                 disable_upgrade_migration: Optional[pulumi.Input[bool]] = None,
                 last_contact_threshold: Optional[pulumi.Input[str]] = None,
                 max_trailing_logs: Optional[pulumi.Input[int]] = None,
                 redundancy_zone_tag: Optional[pulumi.Input[str]] = None,
                 server_stabilization_time: Optional[pulumi.Input[str]] = None,
                 upgrade_version_tag: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = AutopilotConfigArgs.__new__(AutopilotConfigArgs)

            __props__.__dict__["cleanup_dead_servers"] = cleanup_dead_servers
            __props__.__dict__["datacenter"] = datacenter
            __props__.__dict__["disable_upgrade_migration"] = disable_upgrade_migration
            __props__.__dict__["last_contact_threshold"] = last_contact_threshold
            __props__.__dict__["max_trailing_logs"] = max_trailing_logs
            __props__.__dict__["redundancy_zone_tag"] = redundancy_zone_tag
            __props__.__dict__["server_stabilization_time"] = server_stabilization_time
            __props__.__dict__["upgrade_version_tag"] = upgrade_version_tag
        super(AutopilotConfig, __self__).__init__(
            'consul:index/autopilotConfig:AutopilotConfig',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            cleanup_dead_servers: Optional[pulumi.Input[bool]] = None,
            datacenter: Optional[pulumi.Input[str]] = None,
            disable_upgrade_migration: Optional[pulumi.Input[bool]] = None,
            last_contact_threshold: Optional[pulumi.Input[str]] = None,
            max_trailing_logs: Optional[pulumi.Input[int]] = None,
            redundancy_zone_tag: Optional[pulumi.Input[str]] = None,
            server_stabilization_time: Optional[pulumi.Input[str]] = None,
            upgrade_version_tag: Optional[pulumi.Input[str]] = None) -> 'AutopilotConfig':
        """
        Get an existing AutopilotConfig resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] cleanup_dead_servers: Whether to remove failing servers when a
               replacement comes online. Defaults to true.
        :param pulumi.Input[str] datacenter: The datacenter to use. This overrides the agent's
               default datacenter and the datacenter in the provider setup.
        :param pulumi.Input[bool] disable_upgrade_migration: Whether to disable [upgrade migrations](https://www.consul.io/docs/guides/autopilot.html#redundancy-zones).
               Defaults to false.
        :param pulumi.Input[str] last_contact_threshold: The time after which a server is
               considered as unhealthy and will be removed. Defaults to `"200ms"`.
        :param pulumi.Input[int] max_trailing_logs: The maximum number of Raft log entries a
               server can trail the leader. Defaults to 250.
        :param pulumi.Input[str] redundancy_zone_tag: The [redundancy zone](https://www.consul.io/docs/guides/autopilot.html#redundancy-zones)
               tag to use. Consul will try to keep one voting server by zone to take advantage
               of isolated failure domains. Defaults to an empty string.
        :param pulumi.Input[str] server_stabilization_time: The period to wait for a server to be
               healthy and stable before being promoted to a full, voting member. Defaults to
               `"10s"`.
        :param pulumi.Input[str] upgrade_version_tag: The tag to override the version information
               used during a migration. Defaults to an empty string.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _AutopilotConfigState.__new__(_AutopilotConfigState)

        __props__.__dict__["cleanup_dead_servers"] = cleanup_dead_servers
        __props__.__dict__["datacenter"] = datacenter
        __props__.__dict__["disable_upgrade_migration"] = disable_upgrade_migration
        __props__.__dict__["last_contact_threshold"] = last_contact_threshold
        __props__.__dict__["max_trailing_logs"] = max_trailing_logs
        __props__.__dict__["redundancy_zone_tag"] = redundancy_zone_tag
        __props__.__dict__["server_stabilization_time"] = server_stabilization_time
        __props__.__dict__["upgrade_version_tag"] = upgrade_version_tag
        return AutopilotConfig(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="cleanupDeadServers")
    def cleanup_dead_servers(self) -> pulumi.Output[Optional[bool]]:
        """
        Whether to remove failing servers when a
        replacement comes online. Defaults to true.
        """
        return pulumi.get(self, "cleanup_dead_servers")

    @property
    @pulumi.getter
    def datacenter(self) -> pulumi.Output[Optional[str]]:
        """
        The datacenter to use. This overrides the agent's
        default datacenter and the datacenter in the provider setup.
        """
        return pulumi.get(self, "datacenter")

    @property
    @pulumi.getter(name="disableUpgradeMigration")
    def disable_upgrade_migration(self) -> pulumi.Output[Optional[bool]]:
        """
        Whether to disable [upgrade migrations](https://www.consul.io/docs/guides/autopilot.html#redundancy-zones).
        Defaults to false.
        """
        return pulumi.get(self, "disable_upgrade_migration")

    @property
    @pulumi.getter(name="lastContactThreshold")
    def last_contact_threshold(self) -> pulumi.Output[Optional[str]]:
        """
        The time after which a server is
        considered as unhealthy and will be removed. Defaults to `"200ms"`.
        """
        return pulumi.get(self, "last_contact_threshold")

    @property
    @pulumi.getter(name="maxTrailingLogs")
    def max_trailing_logs(self) -> pulumi.Output[Optional[int]]:
        """
        The maximum number of Raft log entries a
        server can trail the leader. Defaults to 250.
        """
        return pulumi.get(self, "max_trailing_logs")

    @property
    @pulumi.getter(name="redundancyZoneTag")
    def redundancy_zone_tag(self) -> pulumi.Output[Optional[str]]:
        """
        The [redundancy zone](https://www.consul.io/docs/guides/autopilot.html#redundancy-zones)
        tag to use. Consul will try to keep one voting server by zone to take advantage
        of isolated failure domains. Defaults to an empty string.
        """
        return pulumi.get(self, "redundancy_zone_tag")

    @property
    @pulumi.getter(name="serverStabilizationTime")
    def server_stabilization_time(self) -> pulumi.Output[Optional[str]]:
        """
        The period to wait for a server to be
        healthy and stable before being promoted to a full, voting member. Defaults to
        `"10s"`.
        """
        return pulumi.get(self, "server_stabilization_time")

    @property
    @pulumi.getter(name="upgradeVersionTag")
    def upgrade_version_tag(self) -> pulumi.Output[Optional[str]]:
        """
        The tag to override the version information
        used during a migration. Defaults to an empty string.
        """
        return pulumi.get(self, "upgrade_version_tag")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = [
    'GetDatacentersResult',
    'AwaitableGetDatacentersResult',
    'get_datacenters',
    'get_datacenters_output',
]

@pulumi.output_type
class GetDatacentersResult:
    """
    A collection of values returned by getDatacenters.
    """
    def __init__(__self__, datacenters=None, id=None):
        if datacenters and not isinstance(datacenters, list):
            raise TypeError("Expected argument 'datacenters' to be a list")
        pulumi.set(__self__, "datacenters", datacenters)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)

    @property
    @pulumi.getter
    def datacenters(self) -> Sequence[str]:
        """
        The list of datacenters known. The datacenters will be sorted
        in ascending order based on the estimated median round trip time from the server
        to the servers in that datacenter.
        """
        return pulumi.get(self, "datacenters")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")


class AwaitableGetDatacentersResult(GetDatacentersResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetDatacentersResult(
            datacenters=self.datacenters,
            id=self.id)


def get_datacenters(opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetDatacentersResult:
    """
    The `get_datacenters` data source returns the list of all knwown Consul
    datacenters.
    """
    __args__ = dict()
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('consul:index/getDatacenters:getDatacenters', __args__, opts=opts, typ=GetDatacentersResult).value

    return AwaitableGetDatacentersResult(
        datacenters=pulumi.get(__ret__, 'datacenters'),
        id=pulumi.get(__ret__, 'id'))
def get_datacenters_output(opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetDatacentersResult]:
    """
    The `get_datacenters` data source returns the list of all knwown Consul
    datacenters.
    """
    __args__ = dict()
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('consul:index/getDatacenters:getDatacenters', __args__, opts=opts, typ=GetDatacentersResult)
    return __ret__.apply(lambda __response__: GetDatacentersResult(
        datacenters=pulumi.get(__response__, 'datacenters'),
        id=pulumi.get(__response__, 'id')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs

__all__ = [
    'GetNetworkAreaMembersResult',
    'AwaitableGetNetworkAreaMembersResult',
    'get_network_area_members',
    'get_network_area_members_output',
]

@pulumi.output_type
class GetNetworkAreaMembersResult:
    """
    A collection of values returned by getNetworkAreaMembers.
    """
    def __init__(__self__, datacenter=None, id=None, members=None, token=None, uuid=None):
        if datacenter and not isinstance(datacenter, str):
            raise TypeError("Expected argument 'datacenter' to be a str")
        pulumi.set(__self__, "datacenter", datacenter)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if members and not isinstance(members, list):
            raise TypeError("Expected argument 'members' to be a list")
        pulumi.set(__self__, "members", members)
        if token and not isinstance(token, str):
            raise TypeError("Expected argument 'token' to be a str")
        pulumi.set(__self__, "token", token)
        if uuid and not isinstance(uuid, str):
            raise TypeError("Expected argument 'uuid' to be a str")
        pulumi.set(__self__, "uuid", uuid)

    @property
    @pulumi.getter
    def datacenter(self) -> str:
        """
        The node's Consul datacenter.
        """
        return pulumi.get(self, "datacenter")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def members(self) -> Sequence['outputs.GetNetworkAreaMembersMemberResult']:
        """
        The list of Consul servers in this network area
        """
        return pulumi.get(self, "members")

    @property
    @pulumi.getter
    @_utilities.deprecated("""The token argument has been deprecated and will be removed in a future release.
Please use the token argument in the provider configuration""")
    def token(self) -> Optional[str]:
        return pulumi.get(self, "token")

    @property
    @pulumi.getter
    def uuid(self) -> str:
        """
        The UUID of the Network Area being queried.
        """
        return pulumi.get(self, "uuid")


class AwaitableGetNetworkAreaMembersResult(GetNetworkAreaMembersResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetNetworkAreaMembersResult(
            datacenter=self.datacenter,
            id=self.id,
            members=self.members,
            token=self.token,
            uuid=self.uuid)


def get_network_area_members(datacenter: Optional[str] = None,
                             token: Optional[str] = None,
                             uuid: Optional[str] = None,
                             opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetNetworkAreaMembersResult:
    """
    > **NOTE:** This feature requires [Consul Enterprise](https://www.consul.io/docs/enterprise/index.html).

    The `get_network_area_members` data source provides a list of the Consul
    servers present in a specific network area.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_consul as consul

    dc2_network_area = consul.NetworkArea("dc2",
        peer_datacenter="dc2",
        retry_joins=["1.2.3.4"],
        use_tls=True)
    dc2 = consul.get_network_area_members_output(uuid=dc2_network_area.id)
    pulumi.export("members", dc2.members)
    ```


    :param str datacenter: The datacenter to use. This overrides the
           agent's default datacenter and the datacenter in the provider setup.
    :param str token: The ACL token to use. This overrides the
           token that the agent provides by default.
    :param str uuid: The UUID of the area to list.
    """
    __args__ = dict()
    __args__['datacenter'] = datacenter
    __args__['token'] = token
    __args__['uuid'] = uuid
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('consul:index/getNetworkAreaMembers:getNetworkAreaMembers', __args__, opts=opts, typ=GetNetworkAreaMembersResult).value

    return AwaitableGetNetworkAreaMembersResult(
        datacenter=pulumi.get(__ret__, 'datacenter'),
        id=pulumi.get(__ret__, 'id'),
        members=pulumi.get(__ret__, 'members'),
        token=pulumi.get(__ret__, 'token'),
        uuid=pulumi.get(__ret__, 'uuid'))
def get_network_area_members_output(datacenter: Optional[pulumi.Input[Optional[str]]] = None,
                                    token: Optional[pulumi.Input[Optional[str]]] = None,
                                    uuid: Optional[pulumi.Input[str]] = None,
                                    opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetNetworkAreaMembersResult]:
    """
    > **NOTE:** This feature requires [Consul Enterprise](https://www.consul.io/docs/enterprise/index.html).

    The `get_network_area_members` data source provides a list of the Consul
    servers present in a specific network area.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_consul as consul

    dc2_network_area = consul.NetworkArea("dc2",
        peer_datacenter="dc2",
        retry_joins=["1.2.3.4"],
        use_tls=True)
    dc2 = consul.get_network_area_members_output(uuid=dc2_network_area.id)
    pulumi.export("members", dc2.members)
    ```


    :param str datacenter: The datacenter to use. This overrides the
           agent's default datacenter and the datacenter in the provider setup.
    :param str token: The ACL token to use. This overrides the
           token that the agent provides by default.
    :param str uuid: The UUID of the area to list.
    """
    __args__ = dict()
    __args__['datacenter'] = datacenter
    __args__['token'] = token
    __args__['uuid'] = uuid
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('consul:index/getNetworkAreaMembers:getNetworkAreaMembers', __args__, opts=opts, typ=GetNetworkAreaMembersResult)
    return __ret__.apply(lambda __response__: GetNetworkAreaMembersResult(
        datacenter=pulumi.get(__response__, 'datacenter'),
        id=pulumi.get(__response__, 'id'),
        members=pulumi.get(__response__, 'members'),
        token=pulumi.get(__response__, 'token'),
        uuid=pulumi.get(__response__, 'uuid')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs

__all__ = [
    'GetAclAuthMethodResult',
    'AwaitableGetAclAuthMethodResult',
    'get_acl_auth_method',
    'get_acl_auth_method_output',
]

@pulumi.output_type
class GetAclAuthMethodResult:
    """
    A collection of values returned by getAclAuthMethod.
    """
    def __init__(__self__, config=None, config_json=None, description=None, display_name=None, id=None, max_token_ttl=None, name=None, namespace=None, namespace_rules=None, partition=None, token_locality=None, type=None):
        if config and not isinstance(config, dict):
            raise TypeError("Expected argument 'config' to be a dict")
        pulumi.set(__self__, "config", config)
        if config_json and not isinstance(config_json, str):
            raise TypeError("Expected argument 'config_json' to be a str")
        pulumi.set(__self__, "config_json", config_json)
        if description and not isinstance(description, str):
            raise TypeError("Expected argument 'description' to be a str")
        pulumi.set(__self__, "description", description)
        if display_name and not isinstance(display_name, str):
            raise TypeError("Expected argument 'display_name' to be a str")
        pulumi.set(__self__, "display_name", display_name)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if max_token_ttl and not isinstance(max_token_ttl, str):
            raise TypeError("Expected argument 'max_token_ttl' to be a str")
        pulumi.set(__self__, "max_token_ttl", max_token_ttl)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if namespace and not isinstance(namespace, str):
            raise TypeError("Expected argument 'namespace' to be a str")
        pulumi.set(__self__, "namespace", namespace)
        if namespace_rules and not isinstance(namespace_rules, list):
            raise TypeError("Expected argument 'namespace_rules' to be a list")
        pulumi.set(__self__, "namespace_rules", namespace_rules)
        if partition and not isinstance(partition, str):
            raise TypeError("Expected argument 'partition' to be a str")
        pulumi.set(__self__, "partition", partition)
        if token_locality and not isinstance(token_locality, str):
            raise TypeError("Expected argument 'token_locality' to be a str")
        pulumi.set(__self__, "token_locality", token_locality)
        if type and not isinstance(type, str):
            raise TypeError("Expected argument 'type' to be a str")
        pulumi.set(__self__, "type", type)

    @property
    @pulumi.getter
    @_utilities.deprecated("""The config attribute is deprecated, please use config_json instead.""")
    def config(self) -> Mapping[str, str]:
        """
        The configuration options of the ACL Auth Method. This attribute is
        deprecated and will be removed in a future version. If the configuration is
        too complex to be represented as a map of strings, it will be blank.
        `config_json` should be used instead.
        """
        return pulumi.get(self, "config")

    @property
    @pulumi.getter(name="configJson")
    def config_json(self) -> str:
        """
        The configuration options of the ACL Auth Method.
        """
        return pulumi.get(self, "config_json")

    @property
    @pulumi.getter
    def description(self) -> str:
        """
        The description of the ACL Auth Method.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> str:
        """
        An optional name to use instead of the name attribute when
        displaying information about this auth method.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="maxTokenTtl")
    def max_token_ttl(self) -> str:
        """
        The maximum life of any token created by this auth method.
        """
        return pulumi.get(self, "max_token_ttl")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def namespace(self) -> Optional[str]:
        return pulumi.get(self, "namespace")

    @property
    @pulumi.getter(name="namespaceRules")
    def namespace_rules(self) -> Sequence['outputs.GetAclAuthMethodNamespaceRuleResult']:
        """
        (Enterprise Only) A set of rules that control which
        namespace tokens created via this auth method will be created within
        """
        return pulumi.get(self, "namespace_rules")

    @property
    @pulumi.getter
    def partition(self) -> Optional[str]:
        return pulumi.get(self, "partition")

    @property
    @pulumi.getter(name="tokenLocality")
    def token_locality(self) -> str:
        """
        The kind of token that this auth method produces. This can
        be either 'local' or 'global'.
        """
        return pulumi.get(self, "token_locality")

    @property
    @pulumi.getter
    def type(self) -> str:
        """
        The type of the ACL Auth Method.
        """
        return pulumi.get(self, "type")


class AwaitableGetAclAuthMethodResult(GetAclAuthMethodResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetAclAuthMethodResult(
            config=self.config,
            config_json=self.config_json,
            description=self.description,
            display_name=self.display_name,
            id=self.id,
            max_token_ttl=self.max_token_ttl,
            name=self.name,
            namespace=self.namespace,
            namespace_rules=self.namespace_rules,
            partition=self.partition,
            token_locality=self.token_locality,
            type=self.type)


def get_acl_auth_method(name: Optional[str] = None,
                        namespace: Optional[str] = None,
                        partition: Optional[str] = None,
                        opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetAclAuthMethodResult:
    """
    The `AclAuthMethod` data source returns the information related to a
    [Consul Auth Method](https://www.consul.io/docs/acl/acl-auth-methods.html).

    ## Example Usage

    ```python
    import pulumi
    import pulumi_consul as consul

    test = consul.get_acl_auth_method(name="minikube")
    pulumi.export("consulAclAuthMethod", test.config)
    ```


    :param str name: The name of the ACL Auth Method.
    :param str namespace: The namespace to lookup the auth method.
    :param str partition: The partition to lookup the auth method.
    """
    __args__ = dict()
    __args__['name'] = name
    __args__['namespace'] = namespace
    __args__['partition'] = partition
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('consul:index/getAclAuthMethod:getAclAuthMethod', __args__, opts=opts, typ=GetAclAuthMethodResult).value

    return AwaitableGetAclAuthMethodResult(
        config=pulumi.get(__ret__, 'config'),
        config_json=pulumi.get(__ret__, 'config_json'),
        description=pulumi.get(__ret__, 'description'),
        display_name=pulumi.get(__ret__, 'display_name'),
        id=pulumi.get(__ret__, 'id'),
        max_token_ttl=pulumi.get(__ret__, 'max_token_ttl'),
        name=pulumi.get(__ret__, 'name'),
        namespace=pulumi.get(__ret__, 'namespace'),
        namespace_rules=pulumi.get(__ret__, 'namespace_rules'),
        partition=pulumi.get(__ret__, 'partition'),
        token_locality=pulumi.get(__ret__, 'token_locality'),
        type=pulumi.get(__ret__, 'type'))
def get_acl_auth_method_output(name: Optional[pulumi.Input[str]] = None,
                               namespace: Optional[pulumi.Input[Optional[str]]] = None,
                               partition: Optional[pulumi.Input[Optional[str]]] = None,
                               opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetAclAuthMethodResult]:
    """
    The `AclAuthMethod` data source returns the information related to a
    [Consul Auth Method](https://www.consul.io/docs/acl/acl-auth-methods.html).

    ## Example Usage

    ```python
    import pulumi
    import pulumi_consul as consul

    test = consul.get_acl_auth_method(name="minikube")
    pulumi.export("consulAclAuthMethod", test.config)
    ```


    :param str name: The name of the ACL Auth Method.
    :param str namespace: The namespace to lookup the auth method.
    :param str partition: The partition to lookup the auth method.
    """
    __args__ = dict()
    __args__['name'] = name
    __args__['namespace'] = namespace
    __args__['partition'] = partition
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('consul:index/getAclAuthMethod:getAclAuthMethod', __args__, opts=opts, typ=GetAclAuthMethodResult)
    return __ret__.apply(lambda __response__: GetAclAuthMethodResult(
        config=pulumi.get(__response__, 'config'),
        config_json=pulumi.get(__response__, 'config_json'),
        description=pulumi.get(__response__, 'description'),
        display_name=pulumi.get(__response__, 'display_name'),
        id=pulumi.get(__response__, 'id'),
        max_token_ttl=pulumi.get(__response__, 'max_token_ttl'),
        name=pulumi.get(__response__, 'name'),
        namespace=pulumi.get(__response__, 'namespace'),
        namespace_rules=pulumi.get(__response__, 'namespace_rules'),
        partition=pulumi.get(__response__, 'partition'),
        token_locality=pulumi.get(__response__, 'token_locality'),
        type=pulumi.get(__response__, 'type')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['PreparedQueryArgs', 'PreparedQuery']

@pulumi.input_type
class PreparedQueryArgs:
    def __init__(__self__, *,
                 service: pulumi.Input[str],
                 connect: Optional[pulumi.Input[bool]] = None,
                 datacenter: Optional[pulumi.Input[str]] = None,
                 dns: Optional[pulumi.Input['PreparedQueryDnsArgs']] = None,
                 failover: Optional[pulumi.Input['PreparedQueryFailoverArgs']] = None,
                 ignore_check_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 near: Optional[pulumi.Input[str]] = None,
                 node_meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 only_passing: Optional[pulumi.Input[bool]] = None,
                 service_meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 session: Optional[pulumi.Input[str]] = None,
                 stored_token: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 template: Optional[pulumi.Input['PreparedQueryTemplateArgs']] = None,
                 token: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a PreparedQuery resource.
        :param pulumi.Input[str] service: The name of the service to query
        :param pulumi.Input[bool] connect: When `true` the prepared query will return connect proxy services for a queried service.  Conditions such as `tags` in the prepared query will be matched against the proxy service. Defaults to false.
        :param pulumi.Input[str] datacenter: The datacenter to use. This overrides the agent's default datacenter and the datacenter in the provider setup.
        :param pulumi.Input['PreparedQueryDnsArgs'] dns: Settings for controlling the DNS response details.
        :param pulumi.Input['PreparedQueryFailoverArgs'] failover: Options for controlling behavior when no healthy nodes are available in the local DC.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] ignore_check_ids: Specifies a list of check IDs that should be ignored when filtering unhealthy instances. This is mostly useful in an emergency or as a temporary measure when a health check is found to be unreliable. Being able to ignore it in centrally-defined queries can be simpler than de-registering the check as an interim solution until the check can be fixed.
        :param pulumi.Input[str] name: The name of the prepared query. Used to identify the prepared query during requests. Can be specified as an empty string to configure the query as a catch-all.
        :param pulumi.Input[str] near: Allows specifying the name of a node to sort results near using Consul's distance sorting and network coordinates. The magic `_agent` value can be used to always sort nearest the node servicing the request.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] node_meta: Specifies a list of user-defined key/value pairs that will be used for filtering the query results to nodes with the given metadata values present.
        :param pulumi.Input[bool] only_passing: When `true`, the prepared query will only return nodes with passing health checks in the result.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] service_meta: Specifies a list of user-defined key/value pairs that will be used for filtering the query results to services with the given metadata values present.
        :param pulumi.Input[str] session: The name of the Consul session to tie this query's lifetime to.  This is an advanced parameter that should not be used without a complete understanding of Consul sessions and the implications of their use (it is recommended to leave this blank in nearly all cases).  If this parameter is omitted the query will not expire.
        :param pulumi.Input[str] stored_token: The ACL token to store with the prepared query. This token will be used by default whenever the query is executed.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] tags: The list of required and/or disallowed tags.  If a tag is in this list it must be present.  If the tag is preceded with a "!" then it is disallowed.
        :param pulumi.Input['PreparedQueryTemplateArgs'] template: Query templating options. This is used to make a single prepared query respond to many different requests
        :param pulumi.Input[str] token: The ACL token to use when saving the prepared query. This overrides the token that the agent provides by default.
        """
        pulumi.set(__self__, "service", service)
        if connect is not None:
            pulumi.set(__self__, "connect", connect)
        if datacenter is not None:
            pulumi.set(__self__, "datacenter", datacenter)
        if dns is not None:
            pulumi.set(__self__, "dns", dns)
        if failover is not None:
            pulumi.set(__self__, "failover", failover)
        if ignore_check_ids is not None:
            pulumi.set(__self__, "ignore_check_ids", ignore_check_ids)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if near is not None:
            pulumi.set(__self__, "near", near)
        if node_meta is not None:
            pulumi.set(__self__, "node_meta", node_meta)
        if only_passing is not None:
            pulumi.set(__self__, "only_passing", only_passing)
        if service_meta is not None:
            pulumi.set(__self__, "service_meta", service_meta)
        if session is not None:
            pulumi.set(__self__, "session", session)
        if stored_token is not None:
            pulumi.set(__self__, "stored_token", stored_token)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if template is not None:
            pulumi.set(__self__, "template", template)
        if token is not None:
            warnings.warn("""The token argument has been deprecated and will be removed in a future release.
Please use the token argument in the provider configuration""", DeprecationWarning)
            pulumi.log.warn("""token is deprecated: The token argument has been deprecated and will be removed in a future release.
Please use the token argument in the provider configuration""")
        if token is not None:
            pulumi.set(__self__, "token", token)

    @property
    @pulumi.getter
    def service(self) -> pulumi.Input[str]:
        """
        The name of the service to query
        """
        return pulumi.get(self, "service")

    @service.setter
    def service(self, value: pulumi.Input[str]):
        pulumi.set(self, "service", value)

    @property
    @pulumi.getter
    def connect(self) -> Optional[pulumi.Input[bool]]:
        """
        When `true` the prepared query will return connect proxy services for a queried service.  Conditions such as `tags` in the prepared query will be matched against the proxy service. Defaults to false.
        """
        return pulumi.get(self, "connect")

    @connect.setter
    def connect(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "connect", value)

    @property
    @pulumi.getter
    def datacenter(self) -> Optional[pulumi.Input[str]]:
        """
        The datacenter to use. This overrides the agent's default datacenter and the datacenter in the provider setup.
        """
        return pulumi.get(self, "datacenter")

    @datacenter.setter
    def datacenter(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "datacenter", value)

    @property
    @pulumi.getter
    def dns(self) -> Optional[pulumi.Input['PreparedQueryDnsArgs']]:
        """
        Settings for controlling the DNS response details.
        """
        return pulumi.get(self, "dns")

    @dns.setter
    def dns(self, value: Optional[pulumi.Input['PreparedQueryDnsArgs']]):
        pulumi.set(self, "dns", value)

    @property
    @pulumi.getter
    def failover(self) -> Optional[pulumi.Input['PreparedQueryFailoverArgs']]:
        """
        Options for controlling behavior when no healthy nodes are available in the local DC.
        """
        return pulumi.get(self, "failover")

    @failover.setter
    def failover(self, value: Optional[pulumi.Input['PreparedQueryFailoverArgs']]):
        pulumi.set(self, "failover", value)

    @property
    @pulumi.getter(name="ignoreCheckIds")
    def ignore_check_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Specifies a list of check IDs that should be ignored when filtering unhealthy instances. This is mostly useful in an emergency or as a temporary measure when a health check is found to be unreliable. Being able to ignore it in centrally-defined queries can be simpler than de-registering the check as an interim solution until the check can be fixed.
        """
        return pulumi.get(self, "ignore_check_ids")

    @ignore_check_ids.setter
    def ignore_check_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "ignore_check_ids", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the prepared query. Used to identify the prepared query during requests. Can be specified as an empty string to configure the query as a catch-all.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def near(self) -> Optional[pulumi.Input[str]]:
        """
        Allows specifying the name of a node to sort results near using Consul's distance sorting and network coordinates. The magic `_agent` value can be used to always sort nearest the node servicing the request.
        """
        return pulumi.get(self, "near")

    @near.setter
    def near(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "near", value)

    @property
    @pulumi.getter(name="nodeMeta")
    def node_meta(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        Specifies a list of user-defined key/value pairs that will be used for filtering the query results to nodes with the given metadata values present.
        """
        return pulumi.get(self, "node_meta")

    @node_meta.setter
    def node_meta(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "node_meta", value)

    @property
    @pulumi.getter(name="onlyPassing")
    def only_passing(self) -> Optional[pulumi.Input[bool]]:
        """
        When `true`, the prepared query will only return nodes with passing health checks in the result.
        """
        return pulumi.get(self, "only_passing")

    @only_passing.setter
    def only_passing(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "only_passing", value)

    @property
    @pulumi.getter(name="serviceMeta")
    def service_meta(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        Specifies a list of user-defined key/value pairs that will be used for filtering the query results to services with the given metadata values present.
        """
        return pulumi.get(self, "service_meta")

    @service_meta.setter
    def service_meta(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "service_meta", value)

    @property
    @pulumi.getter
    def session(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the Consul session to tie this query's lifetime to.  This is an advanced parameter that should not be used without a complete understanding of Consul sessions and the implications of their use (it is recommended to leave this blank in nearly all cases).  If this parameter is omitted the query will not expire.
        """
        return pulumi.get(self, "session")

    @session.setter
    def session(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "session", value)

    @property
    @pulumi.getter(name="storedToken")
    def stored_token(self) -> Optional[pulumi.Input[str]]:
        """
        The ACL token to store with the prepared query. This token will be used by default whenever the query is executed.
        """
        return pulumi.get(self, "stored_token")

    @stored_token.setter
    def stored_token(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "stored_token", value)

    @property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The list of required and/or disallowed tags.  If a tag is in this list it must be present.  If the tag is preceded with a "!" then it is disallowed.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "tags", value)

    @property
    @pulumi.getter
    def template(self) -> Optional[pulumi.Input['PreparedQueryTemplateArgs']]:
        """
        Query templating options. This is used to make a single prepared query respond to many different requests
        """
        return pulumi.get(self, "template")

    @template.setter
    def template(self, value: Optional[pulumi.Input['PreparedQueryTemplateArgs']]):
        pulumi.set(self, "template", value)

    @property
    @pulumi.getter
    @_utilities.deprecated("""The token argument has been deprecated and will be removed in a future release.
Please use the token argument in the provider configuration""")
    def token(self) -> Optional[pulumi.Input[str]]:
        """
        The ACL token to use when saving the prepared query. This overrides the token that the agent provides by default.
        """
        return pulumi.get(self, "token")

    @token.setter
    def token(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "token", value)


@pulumi.input_type
class _PreparedQueryState:
    def __init__(__self__, *,
                 connect: Optional[pulumi.Input[bool]] = None,
                 datacenter: Optional[pulumi.Input[str]] = None,
                 dns: Optional[pulumi.Input['PreparedQueryDnsArgs']] = None,
                 failover: Optional[pulumi.Input['PreparedQueryFailoverArgs']] = None,
                 ignore_check_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 near: Optional[pulumi.Input[str]] = None,
                 node_meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 only_passing: Optional[pulumi.Input[bool]] = None,
                 service: Optional[pulumi.Input[str]] = None,
                 service_meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 session: Optional[pulumi.Input[str]] = None,
                 stored_token: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 template: Optional[pulumi.Input['PreparedQueryTemplateArgs']] = None,
                 token: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering PreparedQuery resources.
        :param pulumi.Input[bool] connect: When `true` the prepared query will return connect proxy services for a queried service.  Conditions such as `tags` in the prepared query will be matched against the proxy service. Defaults to false.
        :param pulumi.Input[str] datacenter: The datacenter to use. This overrides the agent's default datacenter and the datacenter in the provider setup.
        :param pulumi.Input['PreparedQueryDnsArgs'] dns: Settings for controlling the DNS response details.
        :param pulumi.Input['PreparedQueryFailoverArgs'] failover: Options for controlling behavior when no healthy nodes are available in the local DC.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] ignore_check_ids: Specifies a list of check IDs that should be ignored when filtering unhealthy instances. This is mostly useful in an emergency or as a temporary measure when a health check is found to be unreliable. Being able to ignore it in centrally-defined queries can be simpler than de-registering the check as an interim solution until the check can be fixed.
        :param pulumi.Input[str] name: The name of the prepared query. Used to identify the prepared query during requests. Can be specified as an empty string to configure the query as a catch-all.
        :param pulumi.Input[str] near: Allows specifying the name of a node to sort results near using Consul's distance sorting and network coordinates. The magic `_agent` value can be used to always sort nearest the node servicing the request.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] node_meta: Specifies a list of user-defined key/value pairs that will be used for filtering the query results to nodes with the given metadata values present.
        :param pulumi.Input[bool] only_passing: When `true`, the prepared query will only return nodes with passing health checks in the result.
        :param pulumi.Input[str] service: The name of the service to query
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] service_meta: Specifies a list of user-defined key/value pairs that will be used for filtering the query results to services with the given metadata values present.
        :param pulumi.Input[str] session: The name of the Consul session to tie this query's lifetime to.  This is an advanced parameter that should not be used without a complete understanding of Consul sessions and the implications of their use (it is recommended to leave this blank in nearly all cases).  If this parameter is omitted the query will not expire.
        :param pulumi.Input[str] stored_token: The ACL token to store with the prepared query. This token will be used by default whenever the query is executed.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] tags: The list of required and/or disallowed tags.  If a tag is in this list it must be present.  If the tag is preceded with a "!" then it is disallowed.
        :param pulumi.Input['PreparedQueryTemplateArgs'] template: Query templating options. This is used to make a single prepared query respond to many different requests
        :param pulumi.Input[str] token: The ACL token to use when saving the prepared query. This overrides the token that the agent provides by default.
        """
        if connect is not None:
            pulumi.set(__self__, "connect", connect)
        if datacenter is not None:
            pulumi.set(__self__, "datacenter", datacenter)
        if dns is not None:
            pulumi.set(__self__, "dns", dns)
        if failover is not None:
            pulumi.set(__self__, "failover", failover)
        if ignore_check_ids is not None:
            pulumi.set(__self__, "ignore_check_ids", ignore_check_ids)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if near is not None:
            pulumi.set(__self__, "near", near)
        if node_meta is not None:
            pulumi.set(__self__, "node_meta", node_meta)
        if only_passing is not None:
            pulumi.set(__self__, "only_passing", only_passing)
        if service is not None:
            pulumi.set(__self__, "service", service)
        if service_meta is not None:
            pulumi.set(__self__, "service_meta", service_meta)
        if session is not None:
            pulumi.set(__self__, "session", session)
        if stored_token is not None:
            pulumi.set(__self__, "stored_token", stored_token)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if template is not None:
            pulumi.set(__self__, "template", template)
        if token is not None:
            warnings.warn("""The token argument has been deprecated and will be removed in a future release.
Please use the token argument in the provider configuration""", DeprecationWarning)
            pulumi.log.warn("""token is deprecated: The token argument has been deprecated and will be removed in a future release.
Please use the token argument in the provider configuration""")
        if token is not None:
            pulumi.set(__self__, "token", token)

    @property
    @pulumi.getter
    def connect(self) -> Optional[pulumi.Input[bool]]:
        """
        When `true` the prepared query will return connect proxy services for a queried service.  Conditions such as `tags` in the prepared query will be matched against the proxy service. Defaults to false.
        """
        return pulumi.get(self, "connect")

    @connect.setter
    def connect(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "connect", value)

    @property
    @pulumi.getter
    def datacenter(self) -> Optional[pulumi.Input[str]]:
        """
        The datacenter to use. This overrides the agent's default datacenter and the datacenter in the provider setup.
        """
        return pulumi.get(self, "datacenter")

    @datacenter.setter
    def datacenter(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "datacenter", value)

    @property
    @pulumi.getter
    def dns(self) -> Optional[pulumi.Input['PreparedQueryDnsArgs']]:
        """
        Settings for controlling the DNS response details.
        """
        return pulumi.get(self, "dns")

    @dns.setter
    def dns(self, value: Optional[pulumi.Input['PreparedQueryDnsArgs']]):
        pulumi.set(self, "dns", value)

    @property
    @pulumi.getter
    def failover(self) -> Optional[pulumi.Input['PreparedQueryFailoverArgs']]:
        """
        Options for controlling behavior when no healthy nodes are available in the local DC.
        """
        return pulumi.get(self, "failover")

    @failover.setter
    def failover(self, value: Optional[pulumi.Input['PreparedQueryFailoverArgs']]):
        pulumi.set(self, "failover", value)

    @property
    @pulumi.getter(name="ignoreCheckIds")
    def ignore_check_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Specifies a list of check IDs that should be ignored when filtering unhealthy instances. This is mostly useful in an emergency or as a temporary measure when a health check is found to be unreliable. Being able to ignore it in centrally-defined queries can be simpler than de-registering the check as an interim solution until the check can be fixed.
        """
        return pulumi.get(self, "ignore_check_ids")

    @ignore_check_ids.setter
    def ignore_check_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "ignore_check_ids", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the prepared query. Used to identify the prepared query during requests. Can be specified as an empty string to configure the query as a catch-all.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def near(self) -> Optional[pulumi.Input[str]]:
        """
        Allows specifying the name of a node to sort results near using Consul's distance sorting and network coordinates. The magic `_agent` value can be used to always sort nearest the node servicing the request.
        """
        return pulumi.get(self, "near")

    @near.setter
    def near(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "near", value)

    @property
    @pulumi.getter(name="nodeMeta")
    def node_meta(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        Specifies a list of user-defined key/value pairs that will be used for filtering the query results to nodes with the given metadata values present.
        """
        return pulumi.get(self, "node_meta")

    @node_meta.setter
    def node_meta(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "node_meta", value)

    @property
    @pulumi.getter(name="onlyPassing")
    def only_passing(self) -> Optional[pulumi.Input[bool]]:
        """
        When `true`, the prepared query will only return nodes with passing health checks in the result.
        """
        return pulumi.get(self, "only_passing")

    @only_passing.setter
    def only_passing(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "only_passing", value)

    @property
    @pulumi.getter
    def service(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the service to query
        """
        return pulumi.get(self, "service")

    @service.setter
    def service(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "service", value)

    @property
    @pulumi.getter(name="serviceMeta")
    def service_meta(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        Specifies a list of user-defined key/value pairs that will be used for filtering the query results to services with the given metadata values present.
        """
        return pulumi.get(self, "service_meta")

    @service_meta.setter
    def service_meta(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "service_meta", value)

    @property
    @pulumi.getter
    def session(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the Consul session to tie this query's lifetime to.  This is an advanced parameter that should not be used without a complete understanding of Consul sessions and the implications of their use (it is recommended to leave this blank in nearly all cases).  If this parameter is omitted the query will not expire.
        """
        return pulumi.get(self, "session")

    @session.setter
    def session(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "session", value)

    @property
    @pulumi.getter(name="storedToken")
    def stored_token(self) -> Optional[pulumi.Input[str]]:
        """
        The ACL token to store with the prepared query. This token will be used by default whenever the query is executed.
        """
        return pulumi.get(self, "stored_token")

    @stored_token.setter
    def stored_token(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "stored_token", value)

    @property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The list of required and/or disallowed tags.  If a tag is in this list it must be present.  If the tag is preceded with a "!" then it is disallowed.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "tags", value)

    @property
    @pulumi.getter
    def template(self) -> Optional[pulumi.Input['PreparedQueryTemplateArgs']]:
        """
        Query templating options. This is used to make a single prepared query respond to many different requests
        """
        return pulumi.get(self, "template")

    @template.setter
    def template(self, value: Optional[pulumi.Input['PreparedQueryTemplateArgs']]):
        pulumi.set(self, "template", value)

    @property
    @pulumi.getter
    @_utilities.deprecated("""The token argument has been deprecated and will be removed in a future release.
Please use the token argument in the provider configuration""")
    def token(self) -> Optional[pulumi.Input[str]]:
        """
        The ACL token to use when saving the prepared query. This overrides the token that the agent provides by default.
        """
        return pulumi.get(self, "token")

    @token.setter
    def token(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "token", value)


class PreparedQuery(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 connect: Optional[pulumi.Input[bool]] = None,
                 datacenter: Optional[pulumi.Input[str]] = None,
                 dns: Optional[pulumi.Input[Union['PreparedQueryDnsArgs', 'PreparedQueryDnsArgsDict']]] = None,
                 failover: Optional[pulumi.Input[Union['PreparedQueryFailoverArgs', 'PreparedQueryFailoverArgsDict']]] = None,
                 ignore_check_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 near: Optional[pulumi.Input[str]] = None,
                 node_meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 only_passing: Optional[pulumi.Input[bool]] = None,
                 service: Optional[pulumi.Input[str]] = None,
                 service_meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 session: Optional[pulumi.Input[str]] = None,
                 stored_token: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 template: Optional[pulumi.Input[Union['PreparedQueryTemplateArgs', 'PreparedQueryTemplateArgsDict']]] = None,
                 token: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_consul as consul

        # Creates a prepared query myquery.query.consul that finds the nearest
        # healthy myapp.service.consul instance that has the active tag and not
        # the standby tag.
        myapp_query = consul.PreparedQuery("myapp-query",
            name="myquery",
            datacenter="us-central1",
            token="abcd",
            stored_token="wxyz",
            only_passing=True,
            near="_agent",
            service="myapp",
            tags=[
                "active",
                "!standby",
            ],
            failover={
                "nearest_n": 3,
                "datacenters": [
                    "us-west1",
                    "us-east-2",
                    "asia-east1",
                ],
            },
            dns={
                "ttl": "30s",
            })
        # Creates a Prepared Query Template that matches *-near-self.query.consul
        # and finds the nearest service that matches the glob character (e.g.
        # foo-near-self.query.consul will find the nearest healthy foo.service.consul).
        service_near_self = consul.PreparedQuery("service-near-self",
            datacenter="nyc1",
            token="abcd",
            stored_token="wxyz",
            name="",
            only_passing=True,
            connect=True,
            near="_agent",
            template={
                "type": "name_prefix_match",
                "regexp": "^(.*)-near-self$",
            },
            service="${match(1)}",
            failover={
                "nearest_n": 3,
                "datacenters": [
                    "dc2",
                    "dc3",
                    "dc4",
                ],
            },
            dns={
                "ttl": "5m",
            })
        ```

        ## Import

        ```sh
        $ pulumi import consul:index/preparedQuery:PreparedQuery my_service 71ecfb82-717a-4258-b4b6-2fb75144d856
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] connect: When `true` the prepared query will return connect proxy services for a queried service.  Conditions such as `tags` in the prepared query will be matched against the proxy service. Defaults to false.
        :param pulumi.Input[str] datacenter: The datacenter to use. This overrides the agent's default datacenter and the datacenter in the provider setup.
        :param pulumi.Input[Union['PreparedQueryDnsArgs', 'PreparedQueryDnsArgsDict']] dns: Settings for controlling the DNS response details.
        :param pulumi.Input[Union['PreparedQueryFailoverArgs', 'PreparedQueryFailoverArgsDict']] failover: Options for controlling behavior when no healthy nodes are available in the local DC.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] ignore_check_ids: Specifies a list of check IDs that should be ignored when filtering unhealthy instances. This is mostly useful in an emergency or as a temporary measure when a health check is found to be unreliable. Being able to ignore it in centrally-defined queries can be simpler than de-registering the check as an interim solution until the check can be fixed.
        :param pulumi.Input[str] name: The name of the prepared query. Used to identify the prepared query during requests. Can be specified as an empty string to configure the query as a catch-all.
        :param pulumi.Input[str] near: Allows specifying the name of a node to sort results near using Consul's distance sorting and network coordinates. The magic `_agent` value can be used to always sort nearest the node servicing the request.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] node_meta: Specifies a list of user-defined key/value pairs that will be used for filtering the query results to nodes with the given metadata values present.
        :param pulumi.Input[bool] only_passing: When `true`, the prepared query will only return nodes with passing health checks in the result.
        :param pulumi.Input[str] service: The name of the service to query
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] service_meta: Specifies a list of user-defined key/value pairs that will be used for filtering the query results to services with the given metadata values present.
        :param pulumi.Input[str] session: The name of the Consul session to tie this query's lifetime to.  This is an advanced parameter that should not be used without a complete understanding of Consul sessions and the implications of their use (it is recommended to leave this blank in nearly all cases).  If this parameter is omitted the query will not expire.
        :param pulumi.Input[str] stored_token: The ACL token to store with the prepared query. This token will be used by default whenever the query is executed.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] tags: The list of required and/or disallowed tags.  If a tag is in this list it must be present.  If the tag is preceded with a "!" then it is disallowed.
        :param pulumi.Input[Union['PreparedQueryTemplateArgs', 'PreparedQueryTemplateArgsDict']] template: Query templating options. This is used to make a single prepared query respond to many different requests
        :param pulumi.Input[str] token: The ACL token to use when saving the prepared query. This overrides the token that the agent provides by default.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: PreparedQueryArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_consul as consul

        # Creates a prepared query myquery.query.consul that finds the nearest
        # healthy myapp.service.consul instance that has the active tag and not
        # the standby tag.
        myapp_query = consul.PreparedQuery("myapp-query",
            name="myquery",
            datacenter="us-central1",
            token="abcd",
            stored_token="wxyz",
            only_passing=True,
            near="_agent",
            service="myapp",
            tags=[
                "active",
                "!standby",
            ],
            failover={
                "nearest_n": 3,
                "datacenters": [
                    "us-west1",
                    "us-east-2",
                    "asia-east1",
                ],
            },
            dns={
                "ttl": "30s",
            })
        # Creates a Prepared Query Template that matches *-near-self.query.consul
        # and finds the nearest service that matches the glob character (e.g.
        # foo-near-self.query.consul will find the nearest healthy foo.service.consul).
        service_near_self = consul.PreparedQuery("service-near-self",
            datacenter="nyc1",
            token="abcd",
            stored_token="wxyz",
            name="",
            only_passing=True,
            connect=True,
            near="_agent",
            template={
                "type": "name_prefix_match",
                "regexp": "^(.*)-near-self$",
            },
            service="${match(1)}",
            failover={
                "nearest_n": 3,
                "datacenters": [
                    "dc2",
                    "dc3",
                    "dc4",
                ],
            },
            dns={
                "ttl": "5m",
            })
        ```

        ## Import

        ```sh
        $ pulumi import consul:index/preparedQuery:PreparedQuery my_service 71ecfb82-717a-4258-b4b6-2fb75144d856
        ```

        :param str resource_name: The name of the resource.
        :param PreparedQueryArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(PreparedQueryArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 connect: Optional[pulumi.Input[bool]] = None,
                 datacenter: Optional[pulumi.Input[str]] = None,
                 dns: Optional[pulumi.Input[Union['PreparedQueryDnsArgs', 'PreparedQueryDnsArgsDict']]] = None,
                 failover: Optional[pulumi.Input[Union['PreparedQueryFailoverArgs', 'PreparedQueryFailoverArgsDict']]] = None,
                 ignore_check_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 near: Optional[pulumi.Input[str]] = None,
                 node_meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 only_passing: Optional[pulumi.Input[bool]] = None,
                 service: Optional[pulumi.Input[str]] = None,
                 service_meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 session: Optional[pulumi.Input[str]] = None,
                 stored_token: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 template: Optional[pulumi.Input[Union['PreparedQueryTemplateArgs', 'PreparedQueryTemplateArgsDict']]] = None,
                 token: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = PreparedQueryArgs.__new__(PreparedQueryArgs)

            __props__.__dict__["connect"] = connect
            __props__.__dict__["datacenter"] = datacenter
            __props__.__dict__["dns"] = dns
            __props__.__dict__["failover"] = failover
            __props__.__dict__["ignore_check_ids"] = ignore_check_ids
            __props__.__dict__["name"] = name
            __props__.__dict__["near"] = near
            __props__.__dict__["node_meta"] = node_meta
            __props__.__dict__["only_passing"] = only_passing
            if service is None and not opts.urn:
                raise TypeError("Missing required property 'service'")
            __props__.__dict__["service"] = service
            __props__.__dict__["service_meta"] = service_meta
            __props__.__dict__["session"] = session
            __props__.__dict__["stored_token"] = stored_token
            __props__.__dict__["tags"] = tags
            __props__.__dict__["template"] = template
            __props__.__dict__["token"] = None if token is None else pulumi.Output.secret(token)
        secret_opts = pulumi.ResourceOptions(additional_secret_outputs=["token"])
        opts = pulumi.ResourceOptions.merge(opts, secret_opts)
        super(PreparedQuery, __self__).__init__(
            'consul:index/preparedQuery:PreparedQuery',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            connect: Optional[pulumi.Input[bool]] = None,
            datacenter: Optional[pulumi.Input[str]] = None,
            dns: Optional[pulumi.Input[Union['PreparedQueryDnsArgs', 'PreparedQueryDnsArgsDict']]] = None,
            failover: Optional[pulumi.Input[Union['PreparedQueryFailoverArgs', 'PreparedQueryFailoverArgsDict']]] = None,
            ignore_check_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            name: Optional[pulumi.Input[str]] = None,
            near: Optional[pulumi.Input[str]] = None,
            node_meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
            only_passing: Optional[pulumi.Input[bool]] = None,
            service: Optional[pulumi.Input[str]] = None,
            service_meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
            session: Optional[pulumi.Input[str]] = None,
            stored_token: Optional[pulumi.Input[str]] = None,
            tags: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            template: Optional[pulumi.Input[Union['PreparedQueryTemplateArgs', 'PreparedQueryTemplateArgsDict']]] = None,
            token: Optional[pulumi.Input[str]] = None) -> 'PreparedQuery':
        """
        Get an existing PreparedQuery resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] connect: When `true` the prepared query will return connect proxy services for a queried service.  Conditions such as `tags` in the prepared query will be matched against the proxy service. Defaults to false.
        :param pulumi.Input[str] datacenter: The datacenter to use. This overrides the agent's default datacenter and the datacenter in the provider setup.
        :param pulumi.Input[Union['PreparedQueryDnsArgs', 'PreparedQueryDnsArgsDict']] dns: Settings for controlling the DNS response details.
        :param pulumi.Input[Union['PreparedQueryFailoverArgs', 'PreparedQueryFailoverArgsDict']] failover: Options for controlling behavior when no healthy nodes are available in the local DC.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] ignore_check_ids: Specifies a list of check IDs that should be ignored when filtering unhealthy instances. This is mostly useful in an emergency or as a temporary measure when a health check is found to be unreliable. Being able to ignore it in centrally-defined queries can be simpler than de-registering the check as an interim solution until the check can be fixed.
        :param pulumi.Input[str] name: The name of the prepared query. Used to identify the prepared query during requests. Can be specified as an empty string to configure the query as a catch-all.
        :param pulumi.Input[str] near: Allows specifying the name of a node to sort results near using Consul's distance sorting and network coordinates. The magic `_agent` value can be used to always sort nearest the node servicing the request.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] node_meta: Specifies a list of user-defined key/value pairs that will be used for filtering the query results to nodes with the given metadata values present.
        :param pulumi.Input[bool] only_passing: When `true`, the prepared query will only return nodes with passing health checks in the result.
        :param pulumi.Input[str] service: The name of the service to query
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] service_meta: Specifies a list of user-defined key/value pairs that will be used for filtering the query results to services with the given metadata values present.
        :param pulumi.Input[str] session: The name of the Consul session to tie this query's lifetime to.  This is an advanced parameter that should not be used without a complete understanding of Consul sessions and the implications of their use (it is recommended to leave this blank in nearly all cases).  If this parameter is omitted the query will not expire.
        :param pulumi.Input[str] stored_token: The ACL token to store with the prepared query. This token will be used by default whenever the query is executed.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] tags: The list of required and/or disallowed tags.  If a tag is in this list it must be present.  If the tag is preceded with a "!" then it is disallowed.
        :param pulumi.Input[Union['PreparedQueryTemplateArgs', 'PreparedQueryTemplateArgsDict']] template: Query templating options. This is used to make a single prepared query respond to many different requests
        :param pulumi.Input[str] token: The ACL token to use when saving the prepared query. This overrides the token that the agent provides by default.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _PreparedQueryState.__new__(_PreparedQueryState)

        __props__.__dict__["connect"] = connect
        __props__.__dict__["datacenter"] = datacenter
        __props__.__dict__["dns"] = dns
        __props__.__dict__["failover"] = failover
        __props__.__dict__["ignore_check_ids"] = ignore_check_ids
        __props__.__dict__["name"] = name
        __props__.__dict__["near"] = near
        __props__.__dict__["node_meta"] = node_meta
        __props__.__dict__["only_passing"] = only_passing
        __props__.__dict__["service"] = service
        __props__.__dict__["service_meta"] = service_meta
        __props__.__dict__["session"] = session
        __props__.__dict__["stored_token"] = stored_token
        __props__.__dict__["tags"] = tags
        __props__.__dict__["template"] = template
        __props__.__dict__["token"] = token
        return PreparedQuery(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def connect(self) -> pulumi.Output[Optional[bool]]:
        """
        When `true` the prepared query will return connect proxy services for a queried service.  Conditions such as `tags` in the prepared query will be matched against the proxy service. Defaults to false.
        """
        return pulumi.get(self, "connect")

    @property
    @pulumi.getter
    def datacenter(self) -> pulumi.Output[Optional[str]]:
        """
        The datacenter to use. This overrides the agent's default datacenter and the datacenter in the provider setup.
        """
        return pulumi.get(self, "datacenter")

    @property
    @pulumi.getter
    def dns(self) -> pulumi.Output[Optional['outputs.PreparedQueryDns']]:
        """
        Settings for controlling the DNS response details.
        """
        return pulumi.get(self, "dns")

    @property
    @pulumi.getter
    def failover(self) -> pulumi.Output[Optional['outputs.PreparedQueryFailover']]:
        """
        Options for controlling behavior when no healthy nodes are available in the local DC.
        """
        return pulumi.get(self, "failover")

    @property
    @pulumi.getter(name="ignoreCheckIds")
    def ignore_check_ids(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        Specifies a list of check IDs that should be ignored when filtering unhealthy instances. This is mostly useful in an emergency or as a temporary measure when a health check is found to be unreliable. Being able to ignore it in centrally-defined queries can be simpler than de-registering the check as an interim solution until the check can be fixed.
        """
        return pulumi.get(self, "ignore_check_ids")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        The name of the prepared query. Used to identify the prepared query during requests. Can be specified as an empty string to configure the query as a catch-all.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def near(self) -> pulumi.Output[Optional[str]]:
        """
        Allows specifying the name of a node to sort results near using Consul's distance sorting and network coordinates. The magic `_agent` value can be used to always sort nearest the node servicing the request.
        """
        return pulumi.get(self, "near")

    @property
    @pulumi.getter(name="nodeMeta")
    def node_meta(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        """
        Specifies a list of user-defined key/value pairs that will be used for filtering the query results to nodes with the given metadata values present.
        """
        return pulumi.get(self, "node_meta")

    @property
    @pulumi.getter(name="onlyPassing")
    def only_passing(self) -> pulumi.Output[Optional[bool]]:
        """
        When `true`, the prepared query will only return nodes with passing health checks in the result.
        """
        return pulumi.get(self, "only_passing")

    @property
    @pulumi.getter
    def service(self) -> pulumi.Output[str]:
        """
        The name of the service to query
        """
        return pulumi.get(self, "service")

    @property
    @pulumi.getter(name="serviceMeta")
    def service_meta(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        """
        Specifies a list of user-defined key/value pairs that will be used for filtering the query results to services with the given metadata values present.
        """
        return pulumi.get(self, "service_meta")

    @property
    @pulumi.getter
    def session(self) -> pulumi.Output[Optional[str]]:
        """
        The name of the Consul session to tie this query's lifetime to.  This is an advanced parameter that should not be used without a complete understanding of Consul sessions and the implications of their use (it is recommended to leave this blank in nearly all cases).  If this parameter is omitted the query will not expire.
        """
        return pulumi.get(self, "session")

    @property
    @pulumi.getter(name="storedToken")
    def stored_token(self) -> pulumi.Output[Optional[str]]:
        """
        The ACL token to store with the prepared query. This token will be used by default whenever the query is executed.
        """
        return pulumi.get(self, "stored_token")

    @property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        The list of required and/or disallowed tags.  If a tag is in this list it must be present.  If the tag is preceded with a "!" then it is disallowed.
        """
        return pulumi.get(self, "tags")

    @property
    @pulumi.getter
    def template(self) -> pulumi.Output[Optional['outputs.PreparedQueryTemplate']]:
        """
        Query templating options. This is used to make a single prepared query respond to many different requests
        """
        return pulumi.get(self, "template")

    @property
    @pulumi.getter
    @_utilities.deprecated("""The token argument has been deprecated and will be removed in a future release.
Please use the token argument in the provider configuration""")
    def token(self) -> pulumi.Output[Optional[str]]:
        """
        The ACL token to use when saving the prepared query. This overrides the token that the agent provides by default.
        """
        return pulumi.get(self, "token")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = [
    'GetPeeringResult',
    'AwaitableGetPeeringResult',
    'get_peering',
    'get_peering_output',
]

@pulumi.output_type
class GetPeeringResult:
    """
    A collection of values returned by getPeering.
    """
    def __init__(__self__, deleted_at=None, id=None, meta=None, partition=None, peer_ca_pems=None, peer_id=None, peer_name=None, peer_server_addresses=None, peer_server_name=None, state=None):
        if deleted_at and not isinstance(deleted_at, str):
            raise TypeError("Expected argument 'deleted_at' to be a str")
        pulumi.set(__self__, "deleted_at", deleted_at)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if meta and not isinstance(meta, dict):
            raise TypeError("Expected argument 'meta' to be a dict")
        pulumi.set(__self__, "meta", meta)
        if partition and not isinstance(partition, str):
            raise TypeError("Expected argument 'partition' to be a str")
        pulumi.set(__self__, "partition", partition)
        if peer_ca_pems and not isinstance(peer_ca_pems, list):
            raise TypeError("Expected argument 'peer_ca_pems' to be a list")
        pulumi.set(__self__, "peer_ca_pems", peer_ca_pems)
        if peer_id and not isinstance(peer_id, str):
            raise TypeError("Expected argument 'peer_id' to be a str")
        pulumi.set(__self__, "peer_id", peer_id)
        if peer_name and not isinstance(peer_name, str):
            raise TypeError("Expected argument 'peer_name' to be a str")
        pulumi.set(__self__, "peer_name", peer_name)
        if peer_server_addresses and not isinstance(peer_server_addresses, list):
            raise TypeError("Expected argument 'peer_server_addresses' to be a list")
        pulumi.set(__self__, "peer_server_addresses", peer_server_addresses)
        if peer_server_name and not isinstance(peer_server_name, str):
            raise TypeError("Expected argument 'peer_server_name' to be a str")
        pulumi.set(__self__, "peer_server_name", peer_server_name)
        if state and not isinstance(state, str):
            raise TypeError("Expected argument 'state' to be a str")
        pulumi.set(__self__, "state", state)

    @property
    @pulumi.getter(name="deletedAt")
    def deleted_at(self) -> str:
        return pulumi.get(self, "deleted_at")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def meta(self) -> Mapping[str, str]:
        return pulumi.get(self, "meta")

    @property
    @pulumi.getter
    def partition(self) -> Optional[str]:
        return pulumi.get(self, "partition")

    @property
    @pulumi.getter(name="peerCaPems")
    def peer_ca_pems(self) -> Sequence[str]:
        return pulumi.get(self, "peer_ca_pems")

    @property
    @pulumi.getter(name="peerId")
    def peer_id(self) -> str:
        return pulumi.get(self, "peer_id")

    @property
    @pulumi.getter(name="peerName")
    def peer_name(self) -> str:
        return pulumi.get(self, "peer_name")

    @property
    @pulumi.getter(name="peerServerAddresses")
    def peer_server_addresses(self) -> Sequence[str]:
        return pulumi.get(self, "peer_server_addresses")

    @property
    @pulumi.getter(name="peerServerName")
    def peer_server_name(self) -> str:
        return pulumi.get(self, "peer_server_name")

    @property
    @pulumi.getter
    def state(self) -> str:
        return pulumi.get(self, "state")


class AwaitableGetPeeringResult(GetPeeringResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetPeeringResult(
            deleted_at=self.deleted_at,
            id=self.id,
            meta=self.meta,
            partition=self.partition,
            peer_ca_pems=self.peer_ca_pems,
            peer_id=self.peer_id,
            peer_name=self.peer_name,
            peer_server_addresses=self.peer_server_addresses,
            peer_server_name=self.peer_server_name,
            state=self.state)


def get_peering(partition: Optional[str] = None,
                peer_name: Optional[str] = None,
                opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetPeeringResult:
    """
    ## Example Usage

    ```python
    import pulumi
    import pulumi_consul as consul

    basic = consul.get_peering(peer_name="peered-cluster")
    ```
    """
    __args__ = dict()
    __args__['partition'] = partition
    __args__['peerName'] = peer_name
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('consul:index/getPeering:getPeering', __args__, opts=opts, typ=GetPeeringResult).value

    return AwaitableGetPeeringResult(
        deleted_at=pulumi.get(__ret__, 'deleted_at'),
        id=pulumi.get(__ret__, 'id'),
        meta=pulumi.get(__ret__, 'meta'),
        partition=pulumi.get(__ret__, 'partition'),
        peer_ca_pems=pulumi.get(__ret__, 'peer_ca_pems'),
        peer_id=pulumi.get(__ret__, 'peer_id'),
        peer_name=pulumi.get(__ret__, 'peer_name'),
        peer_server_addresses=pulumi.get(__ret__, 'peer_server_addresses'),
        peer_server_name=pulumi.get(__ret__, 'peer_server_name'),
        state=pulumi.get(__ret__, 'state'))
def get_peering_output(partition: Optional[pulumi.Input[Optional[str]]] = None,
                       peer_name: Optional[pulumi.Input[str]] = None,
                       opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetPeeringResult]:
    """
    ## Example Usage

    ```python
    import pulumi
    import pulumi_consul as consul

    basic = consul.get_peering(peer_name="peered-cluster")
    ```
    """
    __args__ = dict()
    __args__['partition'] = partition
    __args__['peerName'] = peer_name
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('consul:index/getPeering:getPeering', __args__, opts=opts, typ=GetPeeringResult)
    return __ret__.apply(lambda __response__: GetPeeringResult(
        deleted_at=pulumi.get(__response__, 'deleted_at'),
        id=pulumi.get(__response__, 'id'),
        meta=pulumi.get(__response__, 'meta'),
        partition=pulumi.get(__response__, 'partition'),
        peer_ca_pems=pulumi.get(__response__, 'peer_ca_pems'),
        peer_id=pulumi.get(__response__, 'peer_id'),
        peer_name=pulumi.get(__response__, 'peer_name'),
        peer_server_addresses=pulumi.get(__response__, 'peer_server_addresses'),
        peer_server_name=pulumi.get(__response__, 'peer_server_name'),
        state=pulumi.get(__response__, 'state')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs

import types

__config__ = pulumi.Config('consul')


class _ExportableConfig(types.ModuleType):
    @property
    def address(self) -> Optional[str]:
        """
        The HTTP(S) API address of the agent to use. Defaults to "127.0.0.1:8500".
        """
        return __config__.get('address')

    @property
    def auth_jwt(self) -> Optional[str]:
        """
        Authenticates to Consul using a JWT authentication method.
        """
        return __config__.get('authJwt')

    @property
    def ca_file(self) -> Optional[str]:
        """
        A path to a PEM-encoded certificate authority used to verify the remote agent's certificate.
        """
        return __config__.get('caFile')

    @property
    def ca_path(self) -> Optional[str]:
        """
        A path to a directory of PEM-encoded certificate authority files to use to check the authenticity of client and server
        connections. Can also be specified with the `CONSUL_CAPATH` environment variable.
        """
        return __config__.get('caPath')

    @property
    def ca_pem(self) -> Optional[str]:
        """
        PEM-encoded certificate authority used to verify the remote agent's certificate.
        """
        return __config__.get('caPem')

    @property
    def cert_file(self) -> Optional[str]:
        """
        A path to a PEM-encoded certificate provided to the remote agent; requires use of `key_file` or `key_pem`.
        """
        return __config__.get('certFile')

    @property
    def cert_pem(self) -> Optional[str]:
        """
        PEM-encoded certificate provided to the remote agent; requires use of `key_file` or `key_pem`.
        """
        return __config__.get('certPem')

    @property
    def datacenter(self) -> Optional[str]:
        """
        The datacenter to use. Defaults to that of the agent.
        """
        return __config__.get('datacenter')

    @property
    def headers(self) -> Optional[str]:
        """
        A configuration block, described below, that provides additional headers to be sent along with all requests to the
        Consul server. This block can be specified multiple times.
        """
        return __config__.get('headers')

    @property
    def http_auth(self) -> Optional[str]:
        """
        HTTP Basic Authentication credentials to be used when communicating with Consul, in the format of either `user` or
        `user:pass`. This may also be specified using the `CONSUL_HTTP_AUTH` environment variable.
        """
        return __config__.get('httpAuth')

    @property
    def insecure_https(self) -> Optional[bool]:
        """
        Boolean value to disable SSL certificate verification; setting this value to true is not recommended for production use.
        Only use this with scheme set to "https".
        """
        return __config__.get_bool('insecureHttps')

    @property
    def key_file(self) -> Optional[str]:
        """
        A path to a PEM-encoded private key, required if `cert_file` or `cert_pem` is specified.
        """
        return __config__.get('keyFile')

    @property
    def key_pem(self) -> Optional[str]:
        """
        PEM-encoded private key, required if `cert_file` or `cert_pem` is specified.
        """
        return __config__.get('keyPem')

    @property
    def namespace(self) -> Optional[str]:
        return __config__.get('namespace')

    @property
    def scheme(self) -> Optional[str]:
        """
        The URL scheme of the agent to use ("http" or "https"). Defaults to "http".
        """
        return __config__.get('scheme')

    @property
    def token(self) -> Optional[str]:
        """
        The ACL token to use by default when making requests to the agent. Can also be specified with `CONSUL_HTTP_TOKEN` or
        `CONSUL_TOKEN` as an environment variable.
        """
        return __config__.get('token')




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = ['PeeringArgs', 'Peering']

@pulumi.input_type
class PeeringArgs:
    def __init__(__self__, *,
                 peer_name: pulumi.Input[str],
                 peering_token: pulumi.Input[str],
                 meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 partition: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a Peering resource.
        :param pulumi.Input[str] peer_name: The name assigned to the peer cluster. The `peer_name` is used to reference the peer cluster in service discovery queries and configuration entries such as `service-intentions`. This field must be a valid DNS hostname label.
        :param pulumi.Input[str] peering_token: The peering token fetched from the peer cluster.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] meta: Specifies KV metadata to associate with the peering. This parameter is not required and does not directly impact the cluster peering process.
        """
        pulumi.set(__self__, "peer_name", peer_name)
        pulumi.set(__self__, "peering_token", peering_token)
        if meta is not None:
            pulumi.set(__self__, "meta", meta)
        if partition is not None:
            pulumi.set(__self__, "partition", partition)

    @property
    @pulumi.getter(name="peerName")
    def peer_name(self) -> pulumi.Input[str]:
        """
        The name assigned to the peer cluster. The `peer_name` is used to reference the peer cluster in service discovery queries and configuration entries such as `service-intentions`. This field must be a valid DNS hostname label.
        """
        return pulumi.get(self, "peer_name")

    @peer_name.setter
    def peer_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "peer_name", value)

    @property
    @pulumi.getter(name="peeringToken")
    def peering_token(self) -> pulumi.Input[str]:
        """
        The peering token fetched from the peer cluster.
        """
        return pulumi.get(self, "peering_token")

    @peering_token.setter
    def peering_token(self, value: pulumi.Input[str]):
        pulumi.set(self, "peering_token", value)

    @property
    @pulumi.getter
    def meta(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        Specifies KV metadata to associate with the peering. This parameter is not required and does not directly impact the cluster peering process.
        """
        return pulumi.get(self, "meta")

    @meta.setter
    def meta(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "meta", value)

    @property
    @pulumi.getter
    def partition(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "partition")

    @partition.setter
    def partition(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "partition", value)


@pulumi.input_type
class _PeeringState:
    def __init__(__self__, *,
                 deleted_at: Optional[pulumi.Input[str]] = None,
                 meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 partition: Optional[pulumi.Input[str]] = None,
                 peer_ca_pems: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 peer_id: Optional[pulumi.Input[str]] = None,
                 peer_name: Optional[pulumi.Input[str]] = None,
                 peer_server_addresses: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 peer_server_name: Optional[pulumi.Input[str]] = None,
                 peering_token: Optional[pulumi.Input[str]] = None,
                 state: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering Peering resources.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] meta: Specifies KV metadata to associate with the peering. This parameter is not required and does not directly impact the cluster peering process.
        :param pulumi.Input[str] peer_name: The name assigned to the peer cluster. The `peer_name` is used to reference the peer cluster in service discovery queries and configuration entries such as `service-intentions`. This field must be a valid DNS hostname label.
        :param pulumi.Input[str] peering_token: The peering token fetched from the peer cluster.
        """
        if deleted_at is not None:
            pulumi.set(__self__, "deleted_at", deleted_at)
        if meta is not None:
            pulumi.set(__self__, "meta", meta)
        if partition is not None:
            pulumi.set(__self__, "partition", partition)
        if peer_ca_pems is not None:
            pulumi.set(__self__, "peer_ca_pems", peer_ca_pems)
        if peer_id is not None:
            pulumi.set(__self__, "peer_id", peer_id)
        if peer_name is not None:
            pulumi.set(__self__, "peer_name", peer_name)
        if peer_server_addresses is not None:
            pulumi.set(__self__, "peer_server_addresses", peer_server_addresses)
        if peer_server_name is not None:
            pulumi.set(__self__, "peer_server_name", peer_server_name)
        if peering_token is not None:
            pulumi.set(__self__, "peering_token", peering_token)
        if state is not None:
            pulumi.set(__self__, "state", state)

    @property
    @pulumi.getter(name="deletedAt")
    def deleted_at(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "deleted_at")

    @deleted_at.setter
    def deleted_at(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "deleted_at", value)

    @property
    @pulumi.getter
    def meta(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        Specifies KV metadata to associate with the peering. This parameter is not required and does not directly impact the cluster peering process.
        """
        return pulumi.get(self, "meta")

    @meta.setter
    def meta(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "meta", value)

    @property
    @pulumi.getter
    def partition(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "partition")

    @partition.setter
    def partition(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "partition", value)

    @property
    @pulumi.getter(name="peerCaPems")
    def peer_ca_pems(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        return pulumi.get(self, "peer_ca_pems")

    @peer_ca_pems.setter
    def peer_ca_pems(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "peer_ca_pems", value)

    @property
    @pulumi.getter(name="peerId")
    def peer_id(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "peer_id")

    @peer_id.setter
    def peer_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "peer_id", value)

    @property
    @pulumi.getter(name="peerName")
    def peer_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name assigned to the peer cluster. The `peer_name` is used to reference the peer cluster in service discovery queries and configuration entries such as `service-intentions`. This field must be a valid DNS hostname label.
        """
        return pulumi.get(self, "peer_name")

    @peer_name.setter
    def peer_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "peer_name", value)

    @property
    @pulumi.getter(name="peerServerAddresses")
    def peer_server_addresses(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        return pulumi.get(self, "peer_server_addresses")

    @peer_server_addresses.setter
    def peer_server_addresses(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "peer_server_addresses", value)

    @property
    @pulumi.getter(name="peerServerName")
    def peer_server_name(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "peer_server_name")

    @peer_server_name.setter
    def peer_server_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "peer_server_name", value)

    @property
    @pulumi.getter(name="peeringToken")
    def peering_token(self) -> Optional[pulumi.Input[str]]:
        """
        The peering token fetched from the peer cluster.
        """
        return pulumi.get(self, "peering_token")

    @peering_token.setter
    def peering_token(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "peering_token", value)

    @property
    @pulumi.getter
    def state(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "state")

    @state.setter
    def state(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "state", value)


class Peering(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 partition: Optional[pulumi.Input[str]] = None,
                 peer_name: Optional[pulumi.Input[str]] = None,
                 peering_token: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        [Cluster Peering](https://www.consul.io/docs/connect/cluster-peering) can be used to create connections between two or more independent clusters so that services deployed to different partitions or datacenters can communicate.

        The `cluster_peering` resource can be used to establish the peering after a peering token has been generated.

        > **Cluster peering is currently in technical preview:** Functionality associated with cluster peering is subject to change. You should never use the technical preview release in secure environments or production scenarios. Features in technical preview may have performance issues, scaling issues, and limited support.

        The functionality described here is available only in Consul version 1.13.0 and later.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_consul as consul

        eu_us = consul.PeeringToken("eu-us", peer_name="eu-cluster")
        eu_us_peering = consul.Peering("eu-us",
            peer_name="eu-cluster",
            peering_token=token["peeringToken"],
            meta={
                "hello": "world",
            })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] meta: Specifies KV metadata to associate with the peering. This parameter is not required and does not directly impact the cluster peering process.
        :param pulumi.Input[str] peer_name: The name assigned to the peer cluster. The `peer_name` is used to reference the peer cluster in service discovery queries and configuration entries such as `service-intentions`. This field must be a valid DNS hostname label.
        :param pulumi.Input[str] peering_token: The peering token fetched from the peer cluster.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: PeeringArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        [Cluster Peering](https://www.consul.io/docs/connect/cluster-peering) can be used to create connections between two or more independent clusters so that services deployed to different partitions or datacenters can communicate.

        The `cluster_peering` resource can be used to establish the peering after a peering token has been generated.

        > **Cluster peering is currently in technical preview:** Functionality associated with cluster peering is subject to change. You should never use the technical preview release in secure environments or production scenarios. Features in technical preview may have performance issues, scaling issues, and limited support.

        The functionality described here is available only in Consul version 1.13.0 and later.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_consul as consul

        eu_us = consul.PeeringToken("eu-us", peer_name="eu-cluster")
        eu_us_peering = consul.Peering("eu-us",
            peer_name="eu-cluster",
            peering_token=token["peeringToken"],
            meta={
                "hello": "world",
            })
        ```

        :param str resource_name: The name of the resource.
        :param PeeringArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(PeeringArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 partition: Optional[pulumi.Input[str]] = None,
                 peer_name: Optional[pulumi.Input[str]] = None,
                 peering_token: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = PeeringArgs.__new__(PeeringArgs)

            __props__.__dict__["meta"] = meta
            __props__.__dict__["partition"] = partition
            if peer_name is None and not opts.urn:
                raise TypeError("Missing required property 'peer_name'")
            __props__.__dict__["peer_name"] = peer_name
            if peering_token is None and not opts.urn:
                raise TypeError("Missing required property 'peering_token'")
            __props__.__dict__["peering_token"] = None if peering_token is None else pulumi.Output.secret(peering_token)
            __props__.__dict__["deleted_at"] = None
            __props__.__dict__["peer_ca_pems"] = None
            __props__.__dict__["peer_id"] = None
            __props__.__dict__["peer_server_addresses"] = None
            __props__.__dict__["peer_server_name"] = None
            __props__.__dict__["state"] = None
        secret_opts = pulumi.ResourceOptions(additional_secret_outputs=["peeringToken"])
        opts = pulumi.ResourceOptions.merge(opts, secret_opts)
        super(Peering, __self__).__init__(
            'consul:index/peering:Peering',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            deleted_at: Optional[pulumi.Input[str]] = None,
            meta: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
            partition: Optional[pulumi.Input[str]] = None,
            peer_ca_pems: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            peer_id: Optional[pulumi.Input[str]] = None,
            peer_name: Optional[pulumi.Input[str]] = None,
            peer_server_addresses: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            peer_server_name: Optional[pulumi.Input[str]] = None,
            peering_token: Optional[pulumi.Input[str]] = None,
            state: Optional[pulumi.Input[str]] = None) -> 'Peering':
        """
        Get an existing Peering resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] meta: Specifies KV metadata to associate with the peering. This parameter is not required and does not directly impact the cluster peering process.
        :param pulumi.Input[str] peer_name: The name assigned to the peer cluster. The `peer_name` is used to reference the peer cluster in service discovery queries and configuration entries such as `service-intentions`. This field must be a valid DNS hostname label.
        :param pulumi.Input[str] peering_token: The peering token fetched from the peer cluster.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _PeeringState.__new__(_PeeringState)

        __props__.__dict__["deleted_at"] = deleted_at
        __props__.__dict__["meta"] = meta
        __props__.__dict__["partition"] = partition
        __props__.__dict__["peer_ca_pems"] = peer_ca_pems
        __props__.__dict__["peer_id"] = peer_id
        __props__.__dict__["peer_name"] = peer_name
        __props__.__dict__["peer_server_addresses"] = peer_server_addresses
        __props__.__dict__["peer_server_name"] = peer_server_name
        __props__.__dict__["peering_token"] = peering_token
        __props__.__dict__["state"] = state
        return Peering(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="deletedAt")
    def deleted_at(self) -> pulumi.Output[str]:
        return pulumi.get(self, "deleted_at")

    @property
    @pulumi.getter
    def meta(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        """
        Specifies KV metadata to associate with the peering. This parameter is not required and does not directly impact the cluster peering process.
        """
        return pulumi.get(self, "meta")

    @property
    @pulumi.getter
    def partition(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "partition")

    @property
    @pulumi.getter(name="peerCaPems")
    def peer_ca_pems(self) -> pulumi.Output[Sequence[str]]:
        return pulumi.get(self, "peer_ca_pems")

    @property
    @pulumi.getter(name="peerId")
    def peer_id(self) -> pulumi.Output[str]:
        return pulumi.get(self, "peer_id")

    @property
    @pulumi.getter(name="peerName")
    def peer_name(self) -> pulumi.Output[str]:
        """
        The name assigned to the peer cluster. The `peer_name` is used to reference the peer cluster in service discovery queries and configuration entries such as `service-intentions`. This field must be a valid DNS hostname label.
        """
        return pulumi.get(self, "peer_name")

    @property
    @pulumi.getter(name="peerServerAddresses")
    def peer_server_addresses(self) -> pulumi.Output[Sequence[str]]:
        return pulumi.get(self, "peer_server_addresses")

    @property
    @pulumi.getter(name="peerServerName")
    def peer_server_name(self) -> pulumi.Output[str]:
        return pulumi.get(self, "peer_server_name")

    @property
    @pulumi.getter(name="peeringToken")
    def peering_token(self) -> pulumi.Output[str]:
        """
        The peering token fetched from the peer cluster.
        """
        return pulumi.get(self, "peering_token")

    @property
    @pulumi.getter
    def state(self) -> pulumi.Output[str]:
        return pulumi.get(self, "state")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities
from . import outputs

__all__ = [
    'GetAclTokenResult',
    'AwaitableGetAclTokenResult',
    'get_acl_token',
    'get_acl_token_output',
]

@pulumi.output_type
class GetAclTokenResult:
    """
    A collection of values returned by getAclToken.
    """
    def __init__(__self__, accessor_id=None, description=None, expiration_time=None, id=None, local=None, namespace=None, node_identities=None, partition=None, policies=None, roles=None, service_identities=None, templated_policies=None):
        if accessor_id and not isinstance(accessor_id, str):
            raise TypeError("Expected argument 'accessor_id' to be a str")
        pulumi.set(__self__, "accessor_id", accessor_id)
        if description and not isinstance(description, str):
            raise TypeError("Expected argument 'description' to be a str")
        pulumi.set(__self__, "description", description)
        if expiration_time and not isinstance(expiration_time, str):
            raise TypeError("Expected argument 'expiration_time' to be a str")
        pulumi.set(__self__, "expiration_time", expiration_time)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if local and not isinstance(local, bool):
            raise TypeError("Expected argument 'local' to be a bool")
        pulumi.set(__self__, "local", local)
        if namespace and not isinstance(namespace, str):
            raise TypeError("Expected argument 'namespace' to be a str")
        pulumi.set(__self__, "namespace", namespace)
        if node_identities and not isinstance(node_identities, list):
            raise TypeError("Expected argument 'node_identities' to be a list")
        pulumi.set(__self__, "node_identities", node_identities)
        if partition and not isinstance(partition, str):
            raise TypeError("Expected argument 'partition' to be a str")
        pulumi.set(__self__, "partition", partition)
        if policies and not isinstance(policies, list):
            raise TypeError("Expected argument 'policies' to be a list")
        pulumi.set(__self__, "policies", policies)
        if roles and not isinstance(roles, list):
            raise TypeError("Expected argument 'roles' to be a list")
        pulumi.set(__self__, "roles", roles)
        if service_identities and not isinstance(service_identities, list):
            raise TypeError("Expected argument 'service_identities' to be a list")
        pulumi.set(__self__, "service_identities", service_identities)
        if templated_policies and not isinstance(templated_policies, list):
            raise TypeError("Expected argument 'templated_policies' to be a list")
        pulumi.set(__self__, "templated_policies", templated_policies)

    @property
    @pulumi.getter(name="accessorId")
    def accessor_id(self) -> str:
        """
        The accessor ID of the ACL token.
        """
        return pulumi.get(self, "accessor_id")

    @property
    @pulumi.getter
    def description(self) -> str:
        """
        The description of the ACL token.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="expirationTime")
    def expiration_time(self) -> str:
        """
        If set this represents the point after which a token should be considered revoked and is eligible for destruction.
        """
        return pulumi.get(self, "expiration_time")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def local(self) -> bool:
        """
        Whether the ACL token is local to the datacenter it was created within.
        """
        return pulumi.get(self, "local")

    @property
    @pulumi.getter
    def namespace(self) -> Optional[str]:
        """
        The namespace to lookup the ACL token.
        """
        return pulumi.get(self, "namespace")

    @property
    @pulumi.getter(name="nodeIdentities")
    def node_identities(self) -> Sequence['outputs.GetAclTokenNodeIdentityResult']:
        """
        The list of node identities attached to the token.
        """
        return pulumi.get(self, "node_identities")

    @property
    @pulumi.getter
    def partition(self) -> Optional[str]:
        """
        The partition to lookup the ACL token.
        """
        return pulumi.get(self, "partition")

    @property
    @pulumi.getter
    def policies(self) -> Sequence['outputs.GetAclTokenPolicyResult']:
        """
        A list of policies associated with the ACL token.
        """
        return pulumi.get(self, "policies")

    @property
    @pulumi.getter
    def roles(self) -> Sequence['outputs.GetAclTokenRoleResult']:
        """
        List of roles linked to the token
        """
        return pulumi.get(self, "roles")

    @property
    @pulumi.getter(name="serviceIdentities")
    def service_identities(self) -> Sequence['outputs.GetAclTokenServiceIdentityResult']:
        """
        The list of service identities attached to the token.
        """
        return pulumi.get(self, "service_identities")

    @property
    @pulumi.getter(name="templatedPolicies")
    def templated_policies(self) -> Sequence['outputs.GetAclTokenTemplatedPolicyResult']:
        """
        The list of templated policies that should be applied to the token.
        """
        return pulumi.get(self, "templated_policies")


class AwaitableGetAclTokenResult(GetAclTokenResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetAclTokenResult(
            accessor_id=self.accessor_id,
            description=self.description,
            expiration_time=self.expiration_time,
            id=self.id,
            local=self.local,
            namespace=self.namespace,
            node_identities=self.node_identities,
            partition=self.partition,
            policies=self.policies,
            roles=self.roles,
            service_identities=self.service_identities,
            templated_policies=self.templated_policies)


def get_acl_token(accessor_id: Optional[str] = None,
                  namespace: Optional[str] = None,
                  partition: Optional[str] = None,
                  opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetAclTokenResult:
    """
    The `AclToken` data source returns the information related to the `AclToken` resource with the exception of its secret ID.

    If you want to get the secret ID associated with a token, use the [`get_acl_token_secret_id` data source](https://www.terraform.io/docs/providers/consul/d/acl_token_secret_id.html).

    ## Example Usage

    ```python
    import pulumi
    import pulumi_consul as consul

    test = consul.get_acl_token(accessor_id="00000000-0000-0000-0000-000000000002")
    pulumi.export("consulAclPolicies", test.policies)
    ```


    :param str accessor_id: The accessor ID of the ACL token.
    :param str namespace: The namespace to lookup the ACL token.
    :param str partition: The partition to lookup the ACL token.
    """
    __args__ = dict()
    __args__['accessorId'] = accessor_id
    __args__['namespace'] = namespace
    __args__['partition'] = partition
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('consul:index/getAclToken:getAclToken', __args__, opts=opts, typ=GetAclTokenResult).value

    return AwaitableGetAclTokenResult(
        accessor_id=pulumi.get(__ret__, 'accessor_id'),
        description=pulumi.get(__ret__, 'description'),
        expiration_time=pulumi.get(__ret__, 'expiration_time'),
        id=pulumi.get(__ret__, 'id'),
        local=pulumi.get(__ret__, 'local'),
        namespace=pulumi.get(__ret__, 'namespace'),
        node_identities=pulumi.get(__ret__, 'node_identities'),
        partition=pulumi.get(__ret__, 'partition'),
        policies=pulumi.get(__ret__, 'policies'),
        roles=pulumi.get(__ret__, 'roles'),
        service_identities=pulumi.get(__ret__, 'service_identities'),
        templated_policies=pulumi.get(__ret__, 'templated_policies'))
def get_acl_token_output(accessor_id: Optional[pulumi.Input[str]] = None,
                         namespace: Optional[pulumi.Input[Optional[str]]] = None,
                         partition: Optional[pulumi.Input[Optional[str]]] = None,
                         opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetAclTokenResult]:
    """
    The `AclToken` data source returns the information related to the `AclToken` resource with the exception of its secret ID.

    If you want to get the secret ID associated with a token, use the [`get_acl_token_secret_id` data source](https://www.terraform.io/docs/providers/consul/d/acl_token_secret_id.html).

    ## Example Usage

    ```python
    import pulumi
    import pulumi_consul as consul

    test = consul.get_acl_token(accessor_id="00000000-0000-0000-0000-000000000002")
    pulumi.export("consulAclPolicies", test.policies)
    ```


    :param str accessor_id: The accessor ID of the ACL token.
    :param str namespace: The namespace to lookup the ACL token.
    :param str partition: The partition to lookup the ACL token.
    """
    __args__ = dict()
    __args__['accessorId'] = accessor_id
    __args__['namespace'] = namespace
    __args__['partition'] = partition
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('consul:index/getAclToken:getAclToken', __args__, opts=opts, typ=GetAclTokenResult)
    return __ret__.apply(lambda __response__: GetAclTokenResult(
        accessor_id=pulumi.get(__response__, 'accessor_id'),
        description=pulumi.get(__response__, 'description'),
        expiration_time=pulumi.get(__response__, 'expiration_time'),
        id=pulumi.get(__response__, 'id'),
        local=pulumi.get(__response__, 'local'),
        namespace=pulumi.get(__response__, 'namespace'),
        node_identities=pulumi.get(__response__, 'node_identities'),
        partition=pulumi.get(__response__, 'partition'),
        policies=pulumi.get(__response__, 'policies'),
        roles=pulumi.get(__response__, 'roles'),
        service_identities=pulumi.get(__response__, 'service_identities'),
        templated_policies=pulumi.get(__response__, 'templated_policies')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = ['AgentServiceArgs', 'AgentService']

@pulumi.input_type
class AgentServiceArgs:
    def __init__(__self__, *,
                 address: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 port: Optional[pulumi.Input[int]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None):
        """
        The set of arguments for constructing a AgentService resource.
        :param pulumi.Input[str] address: The address of the service. Defaults to the
               address of the agent.
        :param pulumi.Input[str] name: The name of the service.
        :param pulumi.Input[int] port: The port of the service.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] tags: A list of values that are opaque to Consul,
               but can be used to distinguish between services or nodes.
        """
        if address is not None:
            pulumi.set(__self__, "address", address)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if port is not None:
            pulumi.set(__self__, "port", port)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)

    @property
    @pulumi.getter
    def address(self) -> Optional[pulumi.Input[str]]:
        """
        The address of the service. Defaults to the
        address of the agent.
        """
        return pulumi.get(self, "address")

    @address.setter
    def address(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "address", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the service.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def port(self) -> Optional[pulumi.Input[int]]:
        """
        The port of the service.
        """
        return pulumi.get(self, "port")

    @port.setter
    def port(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "port", value)

    @property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        A list of values that are opaque to Consul,
        but can be used to distinguish between services or nodes.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "tags", value)


@pulumi.input_type
class _AgentServiceState:
    def __init__(__self__, *,
                 address: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 port: Optional[pulumi.Input[int]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None):
        """
        Input properties used for looking up and filtering AgentService resources.
        :param pulumi.Input[str] address: The address of the service. Defaults to the
               address of the agent.
        :param pulumi.Input[str] name: The name of the service.
        :param pulumi.Input[int] port: The port of the service.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] tags: A list of values that are opaque to Consul,
               but can be used to distinguish between services or nodes.
        """
        if address is not None:
            pulumi.set(__self__, "address", address)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if port is not None:
            pulumi.set(__self__, "port", port)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)

    @property
    @pulumi.getter
    def address(self) -> Optional[pulumi.Input[str]]:
        """
        The address of the service. Defaults to the
        address of the agent.
        """
        return pulumi.get(self, "address")

    @address.setter
    def address(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "address", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the service.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def port(self) -> Optional[pulumi.Input[int]]:
        """
        The port of the service.
        """
        return pulumi.get(self, "port")

    @port.setter
    def port(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "port", value)

    @property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        A list of values that are opaque to Consul,
        but can be used to distinguish between services or nodes.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "tags", value)


class AgentService(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 address: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 port: Optional[pulumi.Input[int]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 __props__=None):
        """
        !> The `AgentService` resource has been deprecated in version 2.0.0 of the provider
        and will be removed in a future release. Please read the [upgrade guide](https://www.terraform.io/docs/providers/consul/guides/upgrading.html#deprecation-of-consul_agent_service)
        for more information.

        Provides access to the agent service data in Consul. This can be used to
        define a service associated with a particular agent. Currently, defining
        health checks for an agent service is not supported.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_consul as consul

        app = consul.AgentService("app",
            address="www.google.com",
            name="google",
            port=80,
            tags=[
                "tag0",
                "tag1",
            ])
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] address: The address of the service. Defaults to the
               address of the agent.
        :param pulumi.Input[str] name: The name of the service.
        :param pulumi.Input[int] port: The port of the service.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] tags: A list of values that are opaque to Consul,
               but can be used to distinguish between services or nodes.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[AgentServiceArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        !> The `AgentService` resource has been deprecated in version 2.0.0 of the provider
        and will be removed in a future release. Please read the [upgrade guide](https://www.terraform.io/docs/providers/consul/guides/upgrading.html#deprecation-of-consul_agent_service)
        for more information.

        Provides access to the agent service data in Consul. This can be used to
        define a service associated with a particular agent. Currently, defining
        health checks for an agent service is not supported.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_consul as consul

        app = consul.AgentService("app",
            address="www.google.com",
            name="google",
            port=80,
            tags=[
                "tag0",
                "tag1",
            ])
        ```

        :param str resource_name: The name of the resource.
        :param AgentServiceArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(AgentServiceArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 address: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 port: Optional[pulumi.Input[int]] = None,
                 tags: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = AgentServiceArgs.__new__(AgentServiceArgs)

            __props__.__dict__["address"] = address
            __props__.__dict__["name"] = name
            __props__.__dict__["port"] = port
            __props__.__dict__["tags"] = tags
        super(AgentService, __self__).__init__(
            'consul:index/agentService:AgentService',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            address: Optional[pulumi.Input[str]] = None,
            name: Optional[pulumi.Input[str]] = None,
            port: Optional[pulumi.Input[int]] = None,
            tags: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None) -> 'AgentService':
        """
        Get an existing AgentService resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] address: The address of the service. Defaults to the
               address of the agent.
        :param pulumi.Input[str] name: The name of the service.
        :param pulumi.Input[int] port: The port of the service.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] tags: A list of values that are opaque to Consul,
               but can be used to distinguish between services or nodes.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _AgentServiceState.__new__(_AgentServiceState)

        __props__.__dict__["address"] = address
        __props__.__dict__["name"] = name
        __props__.__dict__["port"] = port
        __props__.__dict__["tags"] = tags
        return AgentService(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def address(self) -> pulumi.Output[str]:
        """
        The address of the service. Defaults to the
        address of the agent.
        """
        return pulumi.get(self, "address")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        The name of the service.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def port(self) -> pulumi.Output[Optional[int]]:
        """
        The port of the service.
        """
        return pulumi.get(self, "port")

    @property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        A list of values that are opaque to Consul,
        but can be used to distinguish between services or nodes.
        """
        return pulumi.get(self, "tags")

